@@ -22,6 +22,8 @@ The most commonly used commands are:
     dump_tables_and_dfs     dump all tables and dfs of fund dm to s3
     update_stock_factor     update stock factor and factor return
     simple_check            check updating modules simply
+    check_refinancing       check refinancing info
+    check_refinancing_impl  check refinancing implementation info
 '''
 
 
@@ -207,6 +209,8 @@ class SurfingEntrance(object):
         print('start to update stock factor')
         self.update_stock_factor(yesterday, wechat_bot)
         print('update stock factor done')
+        self.check_refinancing(yesterday, wechat_bot)
+        self.check_refinancing_impl(yesterday, wechat_bot)
 
     def calc_fund_estimated_nav(self):
         # Use Jenkins to call this function every hour
@@ -471,6 +475,62 @@ class SurfingEntrance(object):
             wechat_bot = WechatBot()
         wechat_bot.send_update_stock_factor_result(yesterday, failed_factors)
 
+    def check_refinancing(self, yesterday=None, wechat_bot=None):
+        from .data.api.raw import RawDataApi
+
+        if yesterday is None:
+            yesterday = self._get_yesterday()
+
+        df = RawDataApi().get_em_stock_refinancing_by_plan_noticed_date(yesterday, yesterday)
+        if df is None or df.empty:
+            return
+
+        df = df[df.issue_object.notna()]
+        # TODO: 目前key是hard code的
+        key = '高瓴'
+        df = df[df.issue_object.str.contains(key)]
+        if df.empty:
+            return
+        stock_info = RawDataApi().get_em_stock_info(stock_list=df.stock_id.to_list())
+        if stock_info is None:
+            return
+
+        if wechat_bot is None:
+            from .util.wechat_bot import WechatBot
+            wechat_bot = WechatBot()
+        wechat_bot.send_refinancing_key_info(yesterday, stock_info.set_index('stock_id')['name'].to_dict(), key)
+
+    def check_refinancing_impl(self, yesterday=None, wechat_bot=None):
+        from .data.api.raw import RawDataApi
+
+        if yesterday is None:
+            yesterday = self._get_yesterday()
+
+        df = RawDataApi().get_em_stock_refinancing_impl(yesterday, yesterday)
+        if df is None or df.empty:
+            return
+
+        df = df[df.issue_object.notna()]
+        # TODO: 目前key是hard code的
+        key = '高瓴'
+        df = df[df.issue_object.str.contains(key)]
+        if df.empty:
+            return
+        stock_info = RawDataApi().get_em_stock_info(stock_list=df.stock_id.to_list())
+        if stock_info is None:
+            return
+        stock_price = RawDataApi().get_em_stock_price(start_date=yesterday, end_date=yesterday, stock_list=df.stock_id.to_list(), columns=['close'])
+        if stock_price is None:
+            return
+        stock_info = stock_info.set_index('stock_id')[['name']].join([stock_price.drop(columns='datetime').set_index('stock_id'), df.set_index('stock_id').issue_price], how='outer')
+        if stock_info.empty:
+            return
+
+        if wechat_bot is None:
+            from .util.wechat_bot import WechatBot
+            wechat_bot = WechatBot()
+        wechat_bot.send_refinancing_impl_key_info(yesterday, stock_info.to_markdown(), key)
+
 
 if __name__ == "__main__":
     entrance = SurfingEntrance()
