@@ -209,7 +209,6 @@ class BasicDataApi(metaclass=Singleton):
                     query = query.filter(
                         FundNav.datetime == dt,
                     )
-
                 tag_df = pd.read_sql(query.statement, query.session.bind)
                 return tag_df
 
@@ -351,22 +350,17 @@ class BasicDataApi(metaclass=Singleton):
                 print(f'Failed to delete data <err_msg> {e} from {IndexPrice.__tablename__}')
                 return None
 
-    def get_fund_nav_adjusted_nv(self, fund_list, start_date, end_date):
+    def get_fund_nav_with_date_range(self, fund_list, start_date, end_date):
         with BasicDatabaseConnector().managed_session() as db_session:
             try:
                 query = db_session.query(
-                        FundNav.adjusted_net_value,
-                        FundNav.fund_id,
-                        FundNav.datetime,
+                        FundNav
                 ).filter(
                         FundNav.fund_id.in_(fund_list),
                         FundNav.datetime >= start_date,
                         FundNav.datetime <= end_date,
                     )
-
-                tag_df = pd.read_sql(query.statement, query.session.bind)
-                return tag_df
-
+                return pd.read_sql(query.statement, query.session.bind)
             except Exception as e:
                 print(f'Failed to get data <err_msg> {e} from {FundNav.__tablename__}')
 
@@ -1016,3 +1010,108 @@ class BasicDataApi(metaclass=Singleton):
     @classmethod
     def get_fund_info_data(cls, *params):
         return cls.get_model_data(FundInfo, *params)
+
+
+    def get_hedge_fund_info(self, fund_id_list: Tuple[str] = ()):
+        with BasicDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                    HedgeFundInfo,
+                )
+                if fund_id_list:
+                    query = query.filter(
+                        HedgeFundInfo.fund_id.in_(fund_id_list),
+                    )
+                return pd.read_sql(query.statement, query.session.bind)
+            except Exception as e:
+                print(f'failed to get_hedge_fund_info <err_msg> {e} from {HedgeFundInfo.__tablename__}')
+
+
+    def get_hedge_fund_nav(self, fund_id_list: Tuple[str] = ()):
+        with BasicDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                    HedgeFundNAV,
+                )
+                if fund_id_list:
+                    query = query.filter(
+                        HedgeFundNAV.fund_id.in_(fund_id_list),
+                    )
+                return pd.read_sql(query.statement, query.session.bind)
+            except Exception as e:
+                print(f'failed to get_hedge_fund_nav <err_msg> {e} from {HedgeFundNAV.__tablename__}')
+
+
+    def get_fof_info(self, fof_id_list: Tuple[str] = ()):
+        with BasicDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                    FOFInfo,
+                )
+                if fof_id_list:
+                    query = query.filter(
+                        FOFInfo.fof_id.in_(fof_id_list),
+                    )
+                return pd.read_sql(query.statement, query.session.bind)
+            except Exception as e:
+                print(f'failed to get_fof_info <err_msg> {e} from {FOFInfo.__tablename__}')
+
+
+    def get_fof_asset_allocation(self, fof_id_list: Tuple[str] = ()):
+        with BasicDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                    FOFAssetAllocation,
+                )
+                if fof_id_list:
+                    query = query.filter(
+                        FOFAssetAllocation.fof_id.in_(fof_id_list),
+                    )
+                return pd.read_sql(query.statement, query.session.bind)
+            except Exception as e:
+                print(f'failed to get_fof_asset_allocation <err_msg> {e} from {FOFAssetAllocation.__tablename__}')
+
+
+    def get_fof_scale_alteration(self, fof_id_list: Tuple[str] = ()):
+        with BasicDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                    FOFScaleAlteration,
+                )
+                if fof_id_list:
+                    query = query.filter(
+                        FOFScaleAlteration.fof_id.in_(fof_id_list),
+                    )
+                return pd.read_sql(query.statement, query.session.bind)
+            except Exception as e:
+                print(f'failed to get_fof_scale_alteration <err_msg> {e} from {FOFScaleAlteration.__tablename__}')
+
+
+    def get_fof_asset_allocation(self, fof_id_list: Tuple[str] = ()):
+        with BasicDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                    FOFAssetAllocation,
+                )
+                if fof_id_list:
+                    query = query.filter(
+                        FOFAssetAllocation.fof_id.in_(fof_id_list),
+                    )
+                return pd.read_sql(query.statement, query.session.bind)
+            except Exception as e:
+                print(f'failed to get_fof_asset_allocation <err_msg> {e} from {FOFAssetAllocation.__tablename__}')
+
+
+    def get_fof_manually(self, fof_id_list: Tuple[str] = ()):
+        with BasicDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                    FOFManually,
+                )
+                if fof_id_list:
+                    query = query.filter(
+                        FOFManually.fof_id.in_(fof_id_list),
+                    )
+                return pd.read_sql(query.statement, query.session.bind)
+            except Exception as e:
+                print(f'failed to get_fof_manually <err_msg> {e} from {FOFManually.__tablename__}')
