@@ -252,7 +252,7 @@ class DerivedDataApi(metaclass=Singleton):
                 )
                 return pd.read_sql(query.statement, query.session.bind)
             except Exception as e:
-                print(f'Failed to get data <err_msg> {e} from {StyleAnalysisFactorReturn.__tablename__}')
+                print(f'Failed to get data <err_msg> {e} from {BarraCNE5FactorReturn.__tablename__}')
                 return
 
     def get_allocation_distribution(self, version:str=1):
@@ -526,3 +526,32 @@ class DerivedDataApi(metaclass=Singleton):
                 print(f'Failed to get_fund_industry_exposure {e} from {FundIndicatorGroup.__tablename__}')
                 return pd.DataFrame([])
 
+
+    def get_fof_nav(self, fof_id_list: Tuple[str] = ()):
+        with DerivedDatabaseConnector().managed_session() as session:
+            try:
+                query = session.query(
+                    FOFNav,
+                )
+                if fof_id_list:
+                    query = query.filter(
+                        FOFNav.fof_id.in_(fof_id_list),
+                    )
+                return pd.read_sql(query.statement, query.session.bind)
+            except Exception as e:
+                print(f'failed to get_fof_nav <err_msg> {e} from {FOFNav.__tablename__}')
+
+    def delete_fof_nav(self, date_to_delete: datetime.date, fof_id_list: List[str]) -> bool:
+        with DerivedDatabaseConnector().managed_session() as session:
+            try:
+                session.query(
+                    FOFNav
+                ).filter(
+                    FOFNav.fof_id.in_(fof_id_list),
+                    FOFNav.datetime == date_to_delete,
+                ).delete(synchronize_session=False)
+                session.commit()
+                return True
+            except Exception as e:
+                print(f'failed to delete data <err_msg> {e} from {FOFNav.__tablename__}')
+                return False
