@@ -1,6 +1,6 @@
 
 import datetime
-from typing import List, Tuple, Union
+from typing import List, Tuple, Union, Optional
 import pandas as pd
 from sqlalchemy.sql import func
 from sqlalchemy import or_
@@ -314,6 +314,35 @@ class RawDataApi(metaclass=Singleton):
                 print(f'Failed to get data <err_msg> {e} from {OSFundNav.__tablename__}')
                 return None
 
+    def get_oversea_index_price(self, start_date: str = '', end_date: str = '', index_id: Tuple[str] = (), columns: Tuple[str] = ()):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                if not columns:
+                    query = db_session.query(
+                        OSIndexPrice
+                    )
+                else:
+                    query = db_session.query(
+                        OSIndexPrice.codes,
+                        OSIndexPrice.datetime,
+                    ).add_columns(*columns)
+                if index_id:
+                    query = query.filter(
+                        OSIndexPrice.codes.in_(index_id),
+                    )
+                if start_date:
+                    query = query.filter(
+                        OSIndexPrice.datetime >= start_date,
+                    )
+                if end_date:
+                    query = query.filter(
+                        OSIndexPrice.datetime <= end_date,
+                    )
+                return pd.read_sql(query.statement, query.session.bind)
+            except Exception as e:
+                print(f'Failed to get data <err_msg> {e} from {OSIndexPrice.__tablename__}')
+                return None
+
     def get_over_sea_fund_info(self):
         with RawDatabaseConnector().managed_session() as db_session:
             try:
@@ -326,6 +355,67 @@ class RawDataApi(metaclass=Singleton):
                 print('Failed to get data <err_msg> {}'.format(e))
                 return None
 
+    def get_over_sea_fund_benchmark(self):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                    OSFundBenchmark
+                )
+                df = pd.read_sql(query.statement, query.session.bind)
+                return df
+            except Exception as e:
+                print('Failed to get data <err_msg> {}'.format(e))
+                return None
+
+    def get_over_sea_em_stock_info(self):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                    EmOverseaStockInfoUS
+                )
+                df = pd.read_sql(query.statement, query.session.bind)
+                return df
+            except Exception as e:
+                print('Failed to get data <err_msg> {}'.format(e))
+                return None
+
+    def get_over_sea_em_hk_stock_info(self):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                    EmOverseaStockInfoHK
+                )
+                df = pd.read_sql(query.statement, query.session.bind)
+                return df
+            except Exception as e:
+                print('Failed to get data <err_msg> {}'.format(e))
+                return None
+
+
+    def get_over_sea_em_index_info(self):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                    EmOverseaIndexInfo
+                )
+                df = pd.read_sql(query.statement, query.session.bind)
+                return df
+            except Exception as e:
+                print('Failed to get data <err_msg> {}'.format(e))
+                return None
+
+    def get_fund_hold_top10_pos(self):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(
+                    OSFundHoldTop10Pos
+                )
+                df = pd.read_sql(query.statement, query.session.bind)
+                return df
+            except Exception as e:
+                print('Failed to get data <err_msg> {}'.format(e))
+                return None
+
     def delete_em_fund_nav(self, start_date, end_date):
         with RawDatabaseConnector().managed_session() as db_session:
             try:
@@ -1430,6 +1520,38 @@ class RawDataApi(metaclass=Singleton):
             except Exception as e:
                 print(f'Failed to get data <err_msg> {e} from {EmStockRefinancing.__tablename__}')
 
+    def get_em_stock_refinancing_by_plan_noticed_date(self, start_date: str = '', end_date: str = ''):
+        with RawDatabaseConnector().managed_session() as raw_session:
+            try:
+                query = raw_session.query(EmStockRefinancing)
+                if start_date:
+                    query = query.filter(
+                        EmStockRefinancing.PLANNOTICEDDATE >= start_date,
+                    )
+                if end_date:
+                    query = query.filter(
+                        EmStockRefinancing.PLANNOTICEDDATE <= end_date,
+                    )
+                return pd.read_sql(query.statement, query.session.bind)
+            except Exception as e:
+                print(f'Failed to get data <err_msg> {e} from {EmStockRefinancing.__tablename__}')
+
+    def get_em_stock_refinancing_impl(self, start_date: str = '', end_date: str = ''):
+        with RawDatabaseConnector().managed_session() as raw_session:
+            try:
+                query = raw_session.query(EmStockRefinancingImpl)
+                if start_date:
+                    query = query.filter(
+                        EmStockRefinancingImpl.DATE >= start_date,
+                    )
+                if end_date:
+                    query = query.filter(
+                        EmStockRefinancingImpl.DATE <= end_date,
+                    )
+                return pd.read_sql(query.statement, query.session.bind)
+            except Exception as e:
+                print(f'Failed to get data <err_msg> {e} from {EmStockRefinancingImpl.__tablename__}')
+
     def get_em_stock_yearly(self, columns: Tuple[str] = ()):
         with RawDatabaseConnector().managed_session() as db_session:
             try:
@@ -1460,15 +1582,19 @@ class RawDataApi(metaclass=Singleton):
     def get_em_stock_balance(self, stock_id=None, columns: Tuple[str] = ()):
         with RawDatabaseConnector().managed_session() as db_session:
             try:
-                query = db_session.query(
-                    EmStockFSBalanceSheet,
-                )
+                if columns:
+                    query = db_session.query(
+                        EmStockFSBalanceSheet.CODES,
+                        EmStockFSBalanceSheet.DATES,
+                    ).add_columns(*columns)
+                else:
+                    query = db_session.query(
+                        EmStockFSBalanceSheet,
+                    )
                 if stock_id:
                     query = query.filter(
                         EmStockFSBalanceSheet.CODES == stock_id,
                     )
-                if columns:
-                    query = query.add_columns(*columns)
                 return pd.read_sql(query.statement, query.session.bind)
             except Exception as e:
                 print(f'Failed to get_em_stock_balance <err_msg> {e} from {EmStockFSBalanceSheet.__tablename__}')
@@ -1790,4 +1916,174 @@ class RawDataApi(metaclass=Singleton):
 
             except Exception as e:
                 print(f'Failed to get_em_stock_concept_info <err_msg> {e} from {EmStockConceptInfo.__tablename__}')
-                return pd.DataFrame([])
\ No newline at end of file
+                return pd.DataFrame([])
+
+
+    def get_qs_fund_indicator(self, dt=None):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                if not dt:
+                    dt = db_session.query(
+                        func.max(QSOverseaFundIndicator.datetime)
+                    ).one_or_none()
+                    if dt:
+                        dt = dt[0]
+                
+                query = db_session.query(
+                    QSOverseaFundIndicator
+                    ).filter(
+                        QSOverseaFundIndicator.datetime == dt
+                    )
+                df = pd.read_sql(query.statement, query.session.bind).drop(columns=['_update_time'])
+                return df
+
+            except Exception as e:
+                print(f'Failed to QSOverseaFundIndicator <err_msg> {e} from {QSOverseaFundIndicator.__tablename__}')
+                return pd.DataFrame([])
+
+    def get_os_stock_info(self):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(OSStockInfo)
+                df = pd.read_sql(query.statement, query.session.bind).drop(columns=['_update_time'])
+                return df
+
+            except Exception as e:
+                print(f'Failed to get_os_stock_info <err_msg> {e} from {OSStockInfo.__tablename__}')
+                return pd.DataFrame([])
+
+    def get_os_corp_bond_info(self):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(OSCorpBondInfo)
+                df = pd.read_sql(query.statement, query.session.bind).drop(columns=['_update_time'])
+                return df
+
+            except Exception as e:
+                print(f'Failed to get_os_corp_bond_info <err_msg> {e} from {OSCorpBondInfo.__tablename__}')
+                return pd.DataFrame([])
+
+    def get_os_govt_bond_info(self):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(OSGovtBondInfo)
+                df = pd.read_sql(query.statement, query.session.bind).drop(columns=['_update_time'])
+                return df
+
+            except Exception as e:
+                print(f'Failed to get_os_govt_bond_info <err_msg> {e} from {OSGovtBondInfo.__tablename__}')
+                return pd.DataFrame([]) 
+
+    def get_os_mtge_bond_info(self):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(OSMtgeBondInfo)
+                df = pd.read_sql(query.statement, query.session.bind).drop(columns=['_update_time'])
+                return df
+
+            except Exception as e:
+                print(f'Failed to get_mtge_bond_info <err_msg> {e} from {OSMtgeBondInfo.__tablename__}')
+                return pd.DataFrame([])
+
+    def get_os_index_future_info(self):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(OSIndexFutureInfo)
+                df = pd.read_sql(query.statement, query.session.bind).drop(columns=['_update_time'])
+                return df
+
+            except Exception as e:
+                print(f'Failed to get_index_future_info <err_msg> {e} from {OSIndexFutureInfo.__tablename__}')
+                return pd.DataFrame([])
+
+    def get_os_muni_info(self):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(OSMuniInfo)
+                df = pd.read_sql(query.statement, query.session.bind).drop(columns=['_update_time'])
+                return df
+
+            except Exception as e:
+                print(f'Failed to get_muni_info <err_msg> {e} from {OSMuniInfo.__tablename__}')
+                return pd.DataFrame([])
+
+    def get_os_pfd_info(self):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(OSPfdInfo)
+                df = pd.read_sql(query.statement, query.session.bind).drop(columns=['_update_time'])
+                return df
+
+            except Exception as e:
+                print(f'Failed to get_os_pfd_info <err_msg> {e} from {OSPfdInfo.__tablename__}')
+                return pd.DataFrame([])
+    
+    def get_os_comdty_info(self):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(OSComdtyInfo)
+                df = pd.read_sql(query.statement, query.session.bind).drop(columns=['_update_time'])
+                return df
+
+            except Exception as e:
+                print(f'Failed to get_os_comdty_info <err_msg> {e} from {OSComdtyInfo.__tablename__}')
+                return pd.DataFrame([])
+
+    def get_os_district_info(self):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(OSDistrictInfo)
+                df = pd.read_sql(query.statement, query.session.bind).drop(columns=['_update_time'])
+                return df
+
+            except Exception as e:
+                print(f'Failed to get_os_district_info <err_msg> {e} from {OSDistrictInfo.__tablename__}')
+                return pd.DataFrame([])
+
+    def get_os_fund_fin_fac(self):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                query = db_session.query(OSDistrictInfo)
+                df = pd.read_sql(query.statement, query.session.bind).drop(columns=['_update_time'])
+                return df
+
+            except Exception as e:
+                print(f'Failed to get_os_district_info <err_msg> {e} from {OSDistrictInfo.__tablename__}')
+                return pd.DataFrame([])
+
+
+                
+    
+    def get_os_stock_fin_fac(self, *, stock_list: Tuple[str] = (), date_list: Tuple[str] = (), start_date: str = '', end_date: str = '', columns: Tuple[str] = ()):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                if not columns:
+                    query = db_session.query(
+                        EmOverseaStockFinFac
+                    )
+                else:
+                    query = db_session.query(
+                        EmOverseaStockFinFac.stock_code,
+                        EmOverseaStockFinFac.report_date,
+                    ).add_columns(*columns)
+                if stock_list:
+                    query = query.filter(
+                        EmOverseaStockFinFac.stock_code.in_(stock_list)
+                    )
+                if date_list:
+                    query = query.filter(
+                        EmOverseaStockFinFac.report_date.in_(date_list)
+                    )
+                else:
+                    if start_date:
+                        query = query.filter(
+                            EmOverseaStockFinFac.report_date >= start_date,
+                        )
+                    if end_date:
+                        query = query.filter(
+                            EmOverseaStockFinFac.report_date <= end_date,
+                        )
+                return pd.read_sql(query.statement, query.session.bind)
+            except Exception as e:
+                print(f'Failed to get data <err_msg> {e} from {EmOverseaStockFinFac.__tablename__}')
+                return None
\ No newline at end of file
