@@ -4,9 +4,11 @@ import datetime
 import traceback
 from ...api.basic import BasicDataApi
 from ...api.raw import RawDataApi
-from ...api.derived import DerivedDataApi
+# from ...api.derived import DerivedDataApi
 from ...view.derived_models import StyleBox
+from ..raw.raw_data_helper import RawDataHelper
 from .derived_data_helper import DerivedDataHelper
+
 from ....constant import SEMI_UPDATE_DATE_LIST
 
 
@@ -106,7 +108,7 @@ class StyleBoxGenerator:
         self.stock_res.columns = ['x', 'y']
 
     # repo_date数据类型为datetime.date
-    def cal_fund(self, repo_date):
+    def cal_fund(self, repo_date, aux_date):
         fund_list = self._basic_api.get_fund_info()
         fund_list = fund_list[(fund_list.wind_class_1 == '股票型基金') | (fund_list.wind_class_2.isin(['普通股票型基金', '偏股混合型基金', '被动指数型基金', '增强指数型基金', '平衡混合型基金', '灵活配置型基金', '股票多空']))]
         fund_list = fund_list[fund_list.end_date > repo_date]
@@ -114,12 +116,16 @@ class StyleBoxGenerator:
         xres = []
         yres = []
         dres = []
-        dtemp = []
+        # dtemp = []
         repo_str = repo_date.strftime('%Y%m%d')
+        aux_str = aux_date.strftime('%Y%m%d')
         # wind的数据更新起来不太方便，我们手工从Choice下载了这部分数据，所以这里换成了Choice的，但是注意这部分数据没有历史，跑历史的话应该返回去用wind的
         # temp_list = self._raw_api.get_wind_fund_stock_portfolio(start_date=repo_str, end_date=repo_str, wind_fund_list=list(self.flist.keys()))
         # 2020年中报开始使用Choice的数据
         temp_list = self._raw_api.get_em_fund_stock_portfolio(start_date=repo_str, end_date=repo_str)
+        aux_temp_list = self._raw_api.get_em_fund_stock_portfolio(start_date=aux_str, end_date=aux_str)
+        aux_temp_list = aux_temp_list[~aux_temp_list.em_id.isin(set(temp_list.em_id.array))]
+        temp_list = pd.concat([temp_list, aux_temp_list])
         for item, fund_id in self.flist.items():
             # print(item+ ': ' +str(c)+'/'+str(l))
             temp = temp_list[temp_list.em_id == item]
@@ -149,31 +155,23 @@ class StyleBoxGenerator:
             xres.append(xf)
             yres.append(yf)
             dres.append(fund_id)
-            dtemp.append(repo_str)
+            # dtemp.append(repo_str)
         xres_df = pd.DataFrame(xres, index=dres)
         yres_df = pd.DataFrame(yres, index=dres)
-        dtemp = pd.DataFrame(dtemp, index=dres)
+        # dtemp = pd.DataFrame(dtemp, index=dres)
         # print(xres_df, yres_df, dtemp)
         self.fund_res = pd.concat([xres_df, yres_df], axis=1)
-        self.fund_res = pd.concat([dtemp, self.fund_res], axis=1)
+        # self.fund_res = pd.concat([dtemp, self.fund_res], axis=1)
         self.fund_res = self.fund_res.reset_index()
-        self.fund_res.columns = ['fund_id', 'datetime', 'x', 'y']
+        self.fund_res.columns = ['fund_id', 'x', 'y']
 
     def process(self, start_date, end_date):
         failed_tasks = []
         try:
-            end_date_dt = pd.to_datetime(end_date, infer_datetime_format=True)
-            md = end_date_dt.strftime('%m%d')
-            sentry_date = pd.Series(SEMI_UPDATE_DATE_LIST)
-            sentry = sentry_date[sentry_date <= md]
-            if sentry.empty:
-                # 这里取前一年的12月31
-                real_date = datetime.date(end_date_dt.year - 1, 12, 31)
-            else:
-                real_date = pd.to_datetime(str(end_date_dt.year) + sentry.array[-1], infer_datetime_format=True).date()
             self.init(start_date, end_date)
             self.cal_stock()
-            self.cal_fund(real_date)
+            real_date, aux_date = RawDataHelper.get_prev_and_aux_target_dates(end_date, SEMI_UPDATE_DATE_LIST)
+            self.cal_fund(real_date, aux_date)
             # # 获取当前的style box
             # now_df = DerivedDataApi().get_fund_style_box(datetime=real_date)
             # if now_df is not None and not now_df.empty:
@@ -192,6 +190,6 @@ class StyleBoxGenerator:
 
 
 if __name__ == "__main__":
-    date = '20201112'
+    date = '20201231'
     sbg = StyleBoxGenerator(DerivedDataHelper())
     sbg.process(date, date)
