@@ -3,7 +3,9 @@ import numpy as np
 import os
 import datetime
 import traceback
-import requests
+from requests import Session
+from requests.adapters import HTTPAdapter
+from requests.packages.urllib3.util import Retry
 from ...api.raw import RawDataApi
 from ...view.raw_models import YahooIndexPrice
 from .raw_data_helper import RawDataHelper
@@ -13,6 +15,13 @@ class WebRawDataDownloader(object):
 
     def __init__(self, data_helper):
         self._data_helper = data_helper
+        self._session = Session()
+        self._session.mount('https://', HTTPAdapter(
+            max_retries=Retry(total=10, backoff_factor=0.02),
+        ))
+        self._session.mount('http://', HTTPAdapter(
+            max_retries=Retry(total=10, backoff_factor=0.02),
+        ))
 
     def read_csv(self, csv_path):
         return pd.read_csv(csv_path, index_col=1)
@@ -52,12 +61,13 @@ class WebRawDataDownloader(object):
                                       '(KHTML, like Gecko) Version/13.0.5 Safari/605.1.15')}
             filename = ''
             try:
-                r = requests.get(url, headers=headers)
+                r = self._session.get(url, headers=headers, timeout=(15, 60))
                 filename = os.path.join('/tmp', 'cm.xlsx')
                 with open(filename, 'wb') as f:
                     f.write(r.content)
-            except Exception:
-                pass
+            except Exception as e:
+                print(f'[cm_index_price] got error {e}')
+                return False
             df = pd.read_excel(filename)
             # Drop last 2 lines in excel, which are
             #   数据来源：	中国货币网
@@ -90,12 +100,13 @@ class WebRawDataDownloader(object):
             url = 'http://www.csindex.com.cn/uploads/file/autofile/perf/H11073perf.xls'
             filename = ''
             try:
-                r = requests.get(url)
+                r = self._session.get(url, timeout=(15, 60))
                 filename = os.path.join('/tmp', 'H11073perf.xls')
                 with open(filename, 'wb') as f:
                     f.write(r.content)
-            except:
-                pass
+            except Exception as e:
+                print(f'[cxindex_index_price] got error {e}')
+                return False
             df = pd.read_excel(filename)
             df = df[['日期Date', '收盘Close', '涨跌幅(%)Change(%)', '成交量（万元）Volume(10 thousand CNY)',
                 '成交金额（元）Turnover']]
@@ -174,12 +185,13 @@ class WebRawDataDownloader(object):
                 url = url_prefix + url_type + url_postfix
                 filename = ''
                 try:
-                    r = requests.get(url)
+                    r = self._session.get(url, timeout=(15, 60))
                     filename = os.path.join('/tmp', csv_i)
                     with open(filename, 'wb') as f:
                         f.write(r.content)
-                except Exception:
-                    pass
+                except Exception as e:
+                    print(f'[yahoo_index_price] got error {e}')
+                    continue
                 df = pd.read_csv(filename)
                 df = df[['Date', 'Open', 'High', 'Low', 'Close', 'Adj Close', 'Volume']]
                 df.columns = ['datetime', 'open', 'high', 'low', 'close', 'adjclose', 'volume']
