@@ -2,6 +2,7 @@ import pandas as pd
 import traceback
 import json
 import math
+import re
 from sqlalchemy import func
 from .compute_fund_recommendation_collection import FundRecommendationCollectionProcessor
 from ...wrapper.mysql import (RawDatabaseConnector, BasicDatabaseConnector, DerivedDatabaseConnector, 
@@ -429,10 +430,26 @@ class FundDailyCollectionProcessor(object):
             if b is None or b == '':
                 result = None
             else:
+                if multi_sign_list[0] in b and multi_sign_list[1] in b:
+                    b = b.split('+')[0]
                 for mul_sign in multi_sign_list:
                     if mul_sign in b:
-                        result = b.split(mul_sign)[0].replace('收益率','')
-            benchmark_split_chname.append(result)     
+                        result = b.split(mul_sign)[0].replace('收益率','').replace(':','')
+            
+            if result is not None:
+                con = re.search('^\s*[0-9]+%',result)
+                if con is not None:
+                    for mul_sign in multi_sign_list:
+                        if mul_sign in b:
+                            result = b.split('+')[0].split(mul_sign)[1].replace('收益率','')
+            
+            if result is not None and result[0] == '(' and result[-1] == ')':
+                result = result[1:-1]
+            if result is not None and '(' in result and ')' not in result:
+                result = result.replace('(','')
+            if result == 'X':
+                result = b.split('+')[0].split('*')[-1]
+            benchmark_split_chname.append(result) 
         fund_info.loc[:, 'first_benchmark_name'] = benchmark_split_chname
         return fund_info
 
@@ -605,6 +622,7 @@ class FundDailyCollectionProcessor(object):
                 if isinstance(r.top_mng_name,float) and math.isnan(r.top_mng_name):
                     df.loc[fund_id,'top_mng_name'] = _mng_df.loc[fund_id,'fund_manager']
             df = df.reset_index()
+            df.total_score = df.total_score.replace(0, None)
             self.append_data(FundDailyCollection.__tablename__, df)
 
             print(df)
