@@ -40,8 +40,7 @@ class FundRecommendationCollectionProcessor(object):
         else:
             print('没有需要插入的新数据')
     
-    def calc(self):
-
+    def calc_whole_data(self):
         sector_info = DataManager.basic_data(func_name='get_sector_info')
         sector_fund = DataManager.basic_data(func_name='get_sector_index_info')
         fund_info = DataManager.basic_data(func_name='get_fund_info')
@@ -56,17 +55,15 @@ class FundRecommendationCollectionProcessor(object):
         abs_fund_ret_list = fund_info[fund_info.wind_class_2 == '股票多空'].fund_id.tolist()
 
         fund_info['trim_name'] = fund_info.desc_name.map(lambda x : re.subn(r'[ABCDEFR]{1,2}(\(人民币\)|\(美元现汇\)|\(美元现钞\)|1|2|3)?$', '', x)[0])
-        not_include_in_market_fund_list = fund_status[~fund_status['trade_status'].isnull()].fund_id.tolist()
+        self.not_include_in_market_fund_list = DataManager.basic_data(func_name='get_inside_market_funds').fund_id.tolist()
         not_include_close_fund_list = fund_open_info.fund_id.tolist()
         fund_info_money_fund = fund_info[fund_info.wind_class_2 == '货币市场型基金'].copy()
-        not_include_no_buy_funds = fund_info_money_fund[ (fund_info_money_fund.desc_name.str.contains('B'))
+        self.not_include_no_buy_funds = fund_info_money_fund[ (fund_info_money_fund.desc_name.str.contains('B'))
                     | (fund_info_money_fund.desc_name.str.contains('C'))
                     | (fund_info_money_fund.desc_name.str.contains('D'))
                     | (fund_info_money_fund.desc_name.str.contains('E'))
                     | (fund_info_money_fund.desc_name.str.contains('F'))].fund_id.tolist()
 
-        
-
         index_info = index_info.set_index('index_id')
         benchmark_info.loc[:,'index_list'] = benchmark_info.benchmark_s_raw.map(lambda x : self.bench_parse(x))
         benchmark_info.loc[:,'index_name'] = benchmark_info.index_list.map(lambda x : index_info.loc[x].desc_name.tolist())
@@ -99,7 +96,7 @@ class FundRecommendationCollectionProcessor(object):
 
         fund_indicator_score = pd.read_parquet(fund_s3_uri)
         fund_info = fund_info[fund_info.fund_id.isin(fund_indicator_score.reset_index().dropna(subset=['total_score']).fund_id)]
-        ehr_funds = []
+        self.ehr_funds = []
         # 这里itertuples会比iterrows快1个数量级
         for r in fund_info[['desc_name', 'fund_id']].itertuples():
             for type_word, problem_words in EHR_PROBLEM_EKYWORDS.items():
@@ -107,18 +104,18 @@ class FundRecommendationCollectionProcessor(object):
                 for problem_i in problem_words:
                     i = i.replace(problem_i, '')
                 if type_word in i:
-                    ehr_funds.append(r.fund_id)
+                    self.ehr_funds.append(r.fund_id)
 
         fund_info_no_money_fund = fund_info[fund_info.wind_class_2 != '货币市场型基金'].copy()
         fund_info_no_money_fund = fund_info_no_money_fund.groupby('trim_name').last()
-        include_c_type = fund_info_no_money_fund.fund_id.tolist()
+        self.include_c_type = fund_info_no_money_fund.fund_id.tolist()
         
         
         fund_indicator_1 = fund_indicator_score[fund_indicator_score['fund_type'] != 'index']
         fund_indicator_2 = fund_indicator_score[fund_indicator_score.fund_type == 'index'].copy()
-        fund_indicator_2['ret_ability'] = fund_indicator_2.alpha_ability
+        #fund_indicator_2['ret_ability'] = fund_indicator_2.alpha_ability
         fund_indicator = pd.concat([fund_indicator_1,fund_indicator_2],axis=0)
-        fund_indicator_part = fund_indicator[['mdd~history_m','annual_ret~history_m','ret_ability','risk_ability','stable_ability','select_time','select_stock','mng_score','total_score','fund_type']]
+        fund_indicator_part = fund_indicator[['mdd~history_daily','annual_ret~history_m','ret_ability','risk_ability','stable_ability','select_time','select_stock','mng_score','total_score','fund_type']]
 
         mng_info = DataManager.derived_data(func_name='get_fund_manager_info')
         mng_indicator_score = pd.read_parquet(mng_s3_uri)
@@ -151,26 +148,33 @@ class FundRecommendationCollectionProcessor(object):
         final_df.sector_name = final_df.sector_name.map(lambda x : json.dumps(x))
         final_df.index_list = final_df.index_list.map(lambda x : json.dumps(x))
         final_df.index_name = final_df.index_name.map(lambda x : json.dumps(x))
-        final_df = final_df.rename(columns={'mdd~history_m':'mdd','annual_ret~history_m':'annual_ret','name':'mng_name'})
+        final_df = final_df.rename(columns={'mdd~history_daily':'mdd','annual_ret~history_m':'annual_ret','name':'mng_name'})
         final_df = final_df.reset_index()
         final_df.loc[:,'is_abs_ret_fund'] = final_df.fund_id.isin(abs_fund_ret_list)
         final_df.loc[:,'is_conv_bond_fund'] = final_df.fund_id.isin(conv_bond_fund_list)
+        return final_df
+    
+    def calc_filter(self, final_df):
         final_df = final_df[final_df.total_score >= 0]
         final_df_1 = final_df[final_df.wind_class_2 != '货币市场型基金']
         final_df_2 = final_df[final_df.wind_class_2 == '货币市场型基金']
-        final_df_1 = final_df_1[ (final_df_1.fund_id.isin(include_c_type))
-                                 & (~ final_df_1.fund_id.isin(not_include_in_market_fund_list))
-                                 & (~ final_df_1.fund_id.isin(ehr_funds))]
+        final_df_1 = final_df_1[ (final_df_1.fund_id.isin(self.include_c_type))
+                                 & (~ final_df_1.fund_id.isin(self.not_include_in_market_fund_list))
+                                 & (~ final_df_1.fund_id.isin(self.ehr_funds))]
 
-        final_df_2 = final_df_2[(~ final_df_2.fund_id.isin(not_include_in_market_fund_list))
-                                & (~ final_df_2.fund_id.isin(not_include_no_buy_funds))]
+        final_df_2 = final_df_2[(~ final_df_2.fund_id.isin(self.not_include_in_market_fund_list))
+                                & (~ final_df_2.fund_id.isin(self.not_include_no_buy_funds))]
         final_df = pd.concat([final_df_1, final_df_2])
         fund_size = DataManager.basic_data(func_name='get_fund_size_range')
         fund_size_df = fund_size.pivot_table(columns='fund_id',index='datetime',values='size').tail(1).T
         fund_size_df.columns = ['size']
         big_funds = fund_size_df[fund_size_df['size'] > 1e8].index.tolist()                
         final_df = final_df.loc[final_df.fund_id.isin(big_funds)]
-        self.result = final_df
+        return final_df
+
+    def calc(self):
+        final_df = self.calc_whole_data()
+        self.result = self.calc_filter(final_df)
 
     def insert(self):
         self.append_data(FundRecommendationCollection.__tablename__, self.result)
\ No newline at end of file
