@@ -1,9 +1,9 @@
 
 from sqlalchemy import CHAR, Column, Integer, Index, BOOLEAN, text, TEXT, Enum
 from sqlalchemy.ext.declarative import declarative_base
-from sqlalchemy.dialects.mysql import DOUBLE, DATE, TINYINT, DATETIME, MEDIUMTEXT
+from sqlalchemy.dialects.mysql import DOUBLE, DATE, TINYINT, SMALLINT, DATETIME, MEDIUMTEXT
 
-from ...constant import SectorType, IndexPriceSource
+from ...constant import SectorType, IndexPriceSource, HoldingAssetType, FundStatus
 
 
 class Base():
@@ -765,4 +765,115 @@ class FundTag(Base):
     tag_id = Column(CHAR(32), primary_key=True)  #标签id
     fund_id = Column(CHAR(10), primary_key=True) # 基金id
     tag_group_id = Column(CHAR(32), primary_key=True) # 标签组名 source_type_ymd
-    tag_name = Column(CHAR(64)) # 标签名
\ No newline at end of file
+    tag_name = Column(CHAR(64)) # 标签名
+
+
+class HedgeFundNAV(Base):
+    '''私募基金净值'''
+
+    __tablename__ = 'hedge_fund_nav'
+
+    fund_id = Column(CHAR(16), primary_key=True)  # 基金ID
+    datetime = Column(DATE, primary_key=True)  # 日期
+    insert_time = Column(DATETIME, primary_key=True)  # 插入时间
+    calc_date = Column(DATE)  # 计算日期
+    net_asset_value = Column(DOUBLE(asdecimal=False), nullable=False)  # 单位净值
+    acc_unit_value = Column(DOUBLE(asdecimal=False))  # 累计净值
+    v_net_value = Column(DOUBLE(asdecimal=False))  # 虚拟净值
+
+    __table_args__ = (
+        Index('idx_hedge_fund_nav_datetime', 'datetime'),
+    )
+
+
+class HedgeFundInfo(Base):
+    '''私募基金信息'''
+
+    __tablename__ = 'hedge_fund_info'
+
+    fund_id = Column(CHAR(16), primary_key=True)  # 基金ID
+    fund_name = Column(TEXT, nullable=False)  # 基金名称
+    manager_id = Column(CHAR(16), nullable=False)  # 私募基金公司ID
+    water_line = Column(DOUBLE(asdecimal=False), nullable=False)  # 水位线
+    incentive_fee_mode = Column(TEXT, nullable=False)  # 业绩报酬计提方法
+    incentive_fee_ratio = Column(DOUBLE(asdecimal=False), nullable=False)  # 业绩计提比例
+    v_nav_decimals = Column(SMALLINT, nullable=False)  # 虚拟净值精度
+
+
+# class HedgeFundMgrInfo(Base):
+#     '''私募基金公司信息'''
+
+#     __tablename__ = 'hedge_fund_mgr_info'
+
+#     manager_id = Column(CHAR(16), primary_key=True) # 私募基金公司ID
+#     manager_name = Column(TEXT, nullable=False)  # 私募基金公司名称
+
+
+class FOFInfo(Base):
+    '''fof产品信息'''
+
+    __tablename__ = 'fof_info'
+
+    fof_id = Column(CHAR(16), primary_key=True)  # 产品ID
+    datetime = Column(CHAR(16), primary_key=True)  # 数据日期（以支持更新操作）
+    fof_name = Column(TEXT, nullable=False)  # 产品名称
+    admin = Column(TEXT, nullable=False)  # 管理人
+    established_date = Column(DATE, nullable=False)  # 成立日期
+    fof_status = Column(TEXT, nullable=False)  # 申赎状态等
+    subscription_fee = Column(DOUBLE(asdecimal=False), nullable=False)  # 认购费率
+    redemption_fee = Column(DOUBLE(asdecimal=False), nullable=False)  # 赎回费率
+    management_fee = Column(DOUBLE(asdecimal=False), nullable=False)  # 管理费率
+    custodian_fee = Column(DOUBLE(asdecimal=False), nullable=False)  # 托管费率
+    administrative_fee = Column(DOUBLE(asdecimal=False), nullable=False)  # 行政管理费率
+    lock_up_period = Column(TEXT, nullable=False)  # 锁定期
+    incentive_fee_mode = Column(TEXT, nullable=False)  # 业绩报酬计提方法
+    incentive_fee = Column(TEXT, nullable=False)  # 业绩报酬提取比例
+    current_depoist_rate = Column(DOUBLE(asdecimal=False), nullable=False)  # 银行活期存款利率
+    initial_raised_fv = Column(DOUBLE(asdecimal=False), nullable=False)  # 初始募集面值
+    initial_net_value = Column(DOUBLE(asdecimal=False), nullable=False)  # 期初总净值
+
+
+class FOFScaleAlteration(Base):
+    '''fof规模变动记录'''
+
+    __tablename__ = 'fof_scale_alteration'
+
+    fof_id = Column(CHAR(16), primary_key=True)  # 产品ID
+    datetime = Column(DATE, primary_key=True)  # 日期
+    id = Column(Integer, primary_key=True, autoincrement=True)
+    confirmed_date = Column(DATE)  # 确认日期
+    deposited_date = Column(DATE)  # 入账日期
+    amount = Column(DOUBLE(asdecimal=False))  # 被申购金额
+    share = Column(DOUBLE(asdecimal=False))  # 被赎回份额
+
+
+class FOFAssetAllocation(Base):
+    '''fof资产配置记录'''
+
+    __tablename__ = 'fof_asset_allocation'
+
+    fof_id = Column(CHAR(16), primary_key=True)  # 产品ID
+    datetime = Column(DATE, primary_key=True)  # 日期
+    fund_id = Column(CHAR(16), primary_key=True, nullable=False)  # 申购/赎回基金ID
+    id = Column(Integer, primary_key=True, autoincrement=True)
+    asset_type = Column(Enum(HoldingAssetType), nullable=False)  # 资产类型
+    amount = Column(DOUBLE(asdecimal=False))  # 申购金额
+    share = Column(DOUBLE(asdecimal=False))  # 赎回份额
+    status = Column(Enum(FundStatus), nullable=False)  # 状态：在途/完成
+    confirmed_date = Column(DATE)  # 确认日期
+    unit_total = Column(DOUBLE(asdecimal=False))  # 确认份额
+
+
+class FOFManually(Base):
+    '''fof手工校正数据'''
+
+    __tablename__ = 'fof_manually'
+
+    fof_id = Column(CHAR(16), primary_key=True)  # 产品ID
+    datetime = Column(DATE, primary_key=True)  # 日期
+    fee_transfer = Column(DOUBLE(asdecimal=False))  # 费用划拨
+    cd_interest_transfer = Column(DOUBLE(asdecimal=False))  # 银行活期存款利息扣除
+    other_fees = Column(DOUBLE(asdecimal=False))  # 其他费用划拨
+    management_fee_error = Column(DOUBLE(asdecimal=False))  # 管理费误差
+    custodian_fee_error = Column(DOUBLE(asdecimal=False))  # 托管费误差
+    admin_service_fee_error = Column(DOUBLE(asdecimal=False))  # 行政服务费误差
