@@ -180,11 +180,11 @@ class FundIndicatorMonthly(Base):
     fund_id = Column(CHAR(10), primary_key=True)  # 基金ID
     datetime = Column(DATE, primary_key=True)  # 日期
     beta_m = Column(DOUBLE(asdecimal=False), nullable=False)  # beta
-    sharpe_ratio_m = Column(DOUBLE(asdecimal=False), nullable=False)  # 夏普比率
-    treynor_ratio_m = Column(DOUBLE(asdecimal=False), nullable=False)  # 特雷诺比率
-    information_ratio_m = Column(DOUBLE(asdecimal=False), nullable=False)  # 信息比率
+    sharpe_ratio_m = Column(DOUBLE(asdecimal=False))  # 夏普比率
+    treynor_ratio_m = Column(DOUBLE(asdecimal=False))  # 特雷诺比率
+    information_ratio_m = Column(DOUBLE(asdecimal=False))  # 信息比率
     jensen_alpha_m = Column(DOUBLE(asdecimal=False), nullable=False)  # 詹森指数
-    calmar_ratio_m = Column(DOUBLE(asdecimal=False), nullable=False)  # 卡玛比率
+    calmar_ratio_m = Column(DOUBLE(asdecimal=False))  # 卡玛比率
 
     __table_args__ = (
         Index('idx_fund_indicator_monthly_datetime_monthly', 'datetime'),
@@ -683,3 +683,13 @@ class FundIndustryExposure(Base):
     datetime = Column(DATE, primary_key=True) # 日期
     ind_class_type = Column(Enum(IndClassType), nullable=False)  # 行业分类类型
     value = Column(TEXT, nullable=False)  # JSON格式的行业暴露数据
+
+
+class FOFNav(Base):
+    '''fof净值数据'''
+
+    __tablename__ = 'fof_nav'
+
+    fof_id = Column(CHAR(16), primary_key=True)  # 产品ID
+    datetime = Column(DATE, primary_key=True)  # 日期
+    nav = Column(DOUBLE(asdecimal=False), nullable=False)  # 单位净值
