@@ -584,6 +584,11 @@ class CashbookFundBackTestEngine(BasicFundBacktestEngine):
 
         if trades_list:
             self.repair_dict_trade_date(trades_list)
+            fund_ids = []
+            for i in trades_list:
+                fund_ids.append(i.get('fund_id'))
+            fund_ids = list(set(fund_ids))
+            self._dm.confirm_fund_nav_exists(fund_ids)
 
         self._report_helper = ReportHelper()
         self._pending_trades = []
@@ -632,6 +637,7 @@ class CashbookFundBackTestEngine(BasicFundBacktestEngine):
         # init days
         start_date = start_date or self._dm.start_date
         end_date = end_date or self._dm.end_date
+        end_date = min(end_date, (datetime.datetime.now() - datetime.timedelta(days=1)).date())
         start_date = start_date if isinstance(start_date, datetime.date) else datetime.datetime.strptime(start_date,'%Y%m%d').date()
         end_date = end_date if isinstance(end_date, datetime.date) else datetime.datetime.strptime(end_date,'%Y%m%d').date()
 
