@@ -6,16 +6,20 @@ from pprint import pprint
 import platform
 import matplotlib as mpl
 import json
+import re
 from collections import Counter
 from .painter.asset_painter import AssetPainter
 from .painter.fund_painter import FundPainter
 from .painter.hybrid_painter import HybridPainter
 from ...data.struct import AssetWeight, AssetPosition, AssetPrice, AssetValue
 from .asset_helper import TAAHelper
+from ...data.manager import DataManager
 from ...data.manager.manager_fund import FundDataManager
 from ...data.struct import FundTrade, TaaTunerParam, FundPosition, TAAParam
 from .asset_helper import SAAHelper, TAAHelper, FAHelper, TAAStatusMode
 from ...util.calculator import Calculator
+from ...constant import IndClassType
+
 
 CURRENT_PLATFORM = platform.system()
 if CURRENT_PLATFORM == 'Darwin':
@@ -460,12 +464,22 @@ class ReportHelper:
 
     def plot_taa_pct_df(self, dt):
         res = []
+        _res = []
         for index_id, pct_data in TaaTunerParam.POOL.items():
             res.append( self.pct_df.loc[index_id][['datetime',pct_data]].reset_index().pivot_table(index='datetime', columns='index_id')[pct_data])
+            index_val_name = TaaTunerParam.PCT_PAIR[index_id]
+            _df = self.pct_df.loc[index_id][['datetime',index_val_name]].reset_index()
+            _df = _df.pivot_table(index='datetime',columns='index_id')[index_val_name]
+            _res.append(_df)
         pct_df = pd.concat(res, axis=1, sort=True)
-        df = pct_df.loc[dt:]
-        HybridPainter.plot_taa_detail(df)
 
+        index_val_df = pd.concat(_res, axis=1, sort=True)
+        index_val_dic = {k : f'{k}_{v}' for k,v in TaaTunerParam.PCT_PAIR.items()}
+        index_val_df = index_val_df.loc[dt:].rename(columns=index_val_dic)
+        df = pct_df.loc[dt:]
+        HybridPainter.plot_taa_detail(df, '指数估值分位数')
+        HybridPainter.plot_taa_detail(index_val_df, '指数估值')
+        
     @staticmethod
     def get_taa_result(index_id:str, start_date:datetime.date, end_date:datetime.date, taaParam:TAAParam, dm:FundDataManager):
         taa_helper = TAAHelper(taa_params=taaParam)
@@ -762,4 +776,185 @@ class ReportHelper:
         cur_d_sta['beta'] = cur_d['beta']
         cur_d_sta['year_length'] = cur_d['year_length']
         cur_d = cur_d_sta
-        return pd.DataFrame(cur_d.rank(pct=True).loc[top_funds].mean(axis=0)).rename(columns={0:'rank'}).sort_values('rank',ascending=False)
\ No newline at end of file
+        return pd.DataFrame(cur_d.rank(pct=True).loc[top_funds].mean(axis=0)).rename(columns={0:'rank'}).sort_values('rank',ascending=False)
+
+    @staticmethod
+    def samp_period_fund_compare_in_index(index_id,score_select,m,begin_date,today,res_df):
+        # 同期 同板块下跑赢指数基金和持仓基金对比
+        index_price = m.dts.index_price
+        fund_nav = m.dts.fund_nav
+        fund_info = m.dts.fund_info
+        active_fund_info = m.dts.active_fund_info
+        index_info = m.dts.index_info
+        _index_price = index_price.loc[begin_date:today]
+        _index_price = _index_price / _index_price.iloc[0]
+        last_day = _index_price.index.values[-1]
+        _index_ret = pd.DataFrame(data=(_index_price.iloc[-1] / 1 - 1).rename('index_ret'))
+        index_ret = _index_ret.loc[index_id].values[0]
+        score_dic = m._score_manager.score_cache[last_day][index_id][score_select.__dict__[index_id]]
+        score_list = sorted(score_dic.items(), key=lambda x:x[1], reverse=True)
+        score_list = [i[0] for i in score_list]
+        fund_list = score_list[:int(len(score_list)/2)]
+        #fund_list = m._score_manager.score_cache[last_day][index_id][score_select.__dict__[index_id]]
+        fund_score = score_select.__dict__[index_id]
+        if fund_score == 'active':
+            fund_pool = m.dts.all_fund_list
+            fund_pool = active_fund_info[(active_fund_info.fund_id.isin(fund_pool)) & (active_fund_info.index_id == index_id)].fund_id.tolist()
+        else:
+            fund_pool = m.dts.fund_list
+            fund_pool = fund_info[(fund_info.fund_id.isin(fund_pool)) & (fund_info.index_id == index_id)].fund_id.tolist()
+        fund_list = [i for i in fund_list if i in fund_pool]
+        fund_nav_week = fund_nav.loc[begin_date: last_day][fund_list]
+        fund_ret = pd.DataFrame(data = (fund_nav_week.iloc[-1] / fund_nav_week.iloc[0] - 1), columns=['ret'])
+        top_fund_list = fund_ret[fund_ret.ret > index_ret].sort_values('ret',ascending=False).head(6).index.tolist()
+        pos_funds = res_df[res_df.index_id==index_id].index.tolist()
+        top_fund_list = list(set(top_fund_list).union(set(pos_funds)))
+        price_df = _index_price[[index_id]].join(fund_nav_week[top_fund_list])
+        price_df = price_df / price_df.iloc[0]
+        name_dict = fund_info.set_index('fund_id').to_dict()['desc_name']
+        name_dict.update(index_info.set_index('index_id').to_dict()['desc_name'])
+        price_df = price_df.rename(columns=name_dict)
+        indicator = m.dts.index_fund_indicator_pack.loc[index_id].loc[last_day].loc[top_fund_list][['alpha','beta','fund_size','track_err']].reset_index()
+        indicator.loc[:,'rank'] = indicator.fund_id.map(lambda x :score_list.index(x) + 1)
+        indicator['desc_name'] = indicator.fund_id.map(name_dict)
+        indicator = pd.merge(indicator,fund_ret, on='fund_id')
+        indicator = indicator.sort_values('ret',ascending=False)
+        indicator['ret'] = indicator.ret.map(lambda x: str(round(x*10000,1))+'BP')
+        indicator['fund_size'] = indicator.fund_size.map(lambda x: str(round(x/1e8,1)) + '亿')
+        indicator['alpha'] = indicator.alpha.map(lambda x: round(x,3))
+        indicator['beta'] = indicator.beta.map(lambda x: round(x,3))
+        indicator['track_err'] = indicator.track_err.map(lambda x: round(x,3))
+        FundPainter.plot_legend_aside(price_df)
+        return indicator
+
+    @staticmethod
+    def port_penetraded_analysis(m, fund_type,res_df):
+        # 组合持仓穿透分析
+        fund_info = m.dts.fund_info
+        fund_list = res_df.index.tolist()
+        
+        stock_weights = DataManager.basic_data(func_name=f'get_fund_hold_{fund_type}_latest', fund_list = fund_list)
+        res_df['weight'] = res_df['weight'].map(lambda x : float(x[:-1]))
+        stock_weights = stock_weights.set_index('fund_id').drop(columns=['datetime','_update_time'])
+        weight_list = [i for i in stock_weights if 'weight' in i]
+        for r in stock_weights.iterrows():
+            fund_id = r[0]
+            fund_weight = res_df.loc[fund_id, 'weight'] /100
+            for weight_i in weight_list:
+                stock_weights.loc[fund_id, weight_i] = stock_weights.loc[fund_id, weight_i] / 100 * fund_weight
+        _df = stock_weights.join(res_df)
+        stock_weights = _df[~_df.index_id.isin(['mmf','sp500rmb'])]
+        _res = []
+        for r in stock_weights.iterrows():
+            fund_id = r[0]
+            for i in range(1,11):
+                dic = {
+                    'fund_id' : fund_id,
+                    f'{fund_type}_name':r[1][f'rank{i}_{fund_type}'],
+                    f'{fund_type}_weight':r[1][f'rank{i}_{fund_type}weight'], 
+                }
+                _res.append(dic)
+        _df_before_agg = pd.DataFrame(_res)
+        _df_before_agg = pd.merge(_df_before_agg,fund_info[['fund_id','desc_name']],on='fund_id')
+        if fund_type == 'bond':
+            _df_before_agg = _df_before_agg.dropna(subset=['bond_name'])
+            rule = '[a-zA-Z0-9’!"#$%&\'()*+,-./:;<=>?@，。?★、…【】《》？“”‘’！[\\]^_`{|}~]+'
+            _df_before_agg['subject'] = _df_before_agg.bond_name.map(lambda x: re.sub(rule, '', x) )
+            _df_before_agg = _df_before_agg.rename(columns={'bond_name':'bond_full_name'})
+            bond_info = DataManager.raw_data(func_name='get_em_bond_info')
+            bond_info['subject'] = bond_info.name.map(lambda x: re.sub(rule, '', x))
+            stock_info = DataManager.raw_data(func_name='get_em_stock_info')
+            conv_bond_info = DataManager.raw_data(func_name='get_em_conv_bond_info')
+            conv_bond_info['subject'] = conv_bond_info.CBNAME.map(lambda x: re.sub(rule, '', x))
+            _df_before_agg = _df_before_agg.rename(columns={'bond_name':'bond_full_name','bond_short_name':'bond_name'})
+            _df_before_agg = _df_before_agg[['subject','bond_weight','desc_name','bond_full_name']].groupby('subject').agg({'bond_weight':'sum',
+                                                                         'desc_name':lambda x: x.tolist(),
+                                                                         'bond_full_name':lambda x: x.tolist()})
+            _df_before_agg = _df_before_agg.join(bond_info[['subject','em2_type']].set_index('subject'))
+            _df_before_agg['desc_name'] = _df_before_agg.desc_name.map(lambda x : list(set(x))) 
+            _df_before_agg = _df_before_agg.reset_index().drop_duplicates('subject').set_index('subject')  
+            _df_before_agg = _df_before_agg.join(bond_info[['subject','is_city_bond']].sort_values('subject',ascending=False).drop_duplicates(subset=['subject']).set_index('subject'))
+            _df_before_agg = _df_before_agg.join(bond_info[['subject','cbl2_type']].drop_duplicates(subset=['subject']).set_index('subject'))
+            
+
+
+        else:
+            _df_before_agg = _df_before_agg.groupby(f'{fund_type}_name').agg({'desc_name':lambda x: x.tolist(),f'{fund_type}_weight':'sum'})
+
+        _sum = _df_before_agg[f'{fund_type}_weight'].sum()
+        _df_before_agg[f'{fund_type}_weight'] = _df_before_agg[f'{fund_type}_weight'] / _sum
+        _df_before_agg = _df_before_agg.sort_values(f'{fund_type}_weight',ascending=False)
+
+
+        if fund_type == 'stock':      
+            _df_before_agg[f'{fund_type}_weight'] = _df_before_agg[f'{fund_type}_weight'].map(lambda x : str(round(100*x,1))+'%')
+            stock_info = DataManager.raw_data(func_name='get_em_stock_info')
+            industry_info = DataManager.raw_data(func_name='get_em_industry_info')
+            industry_info = industry_info[industry_info.ind_class_type == IndClassType.SWI1]
+            stock_info.loc[:,'sw_indus_1'] = stock_info.bl_sws_ind_code.map(lambda x: x.split('-')[0] if x is not None else None)
+            _df1 = stock_info[['stock_id','name','sw_indus_1']]
+            _df2 = industry_info[['em_id','ind_name']].rename(columns={'em_id':'sw_indus_1'})
+            stock_info = pd.merge(_df1,_df2,on='sw_indus_1')
+            _df_before_agg = _df_before_agg.join(stock_info.set_index('name')[['ind_name']]).reset_index().set_index(['ind_name','stock_name'])
+            return _df_before_agg, pd.DataFrame()
+        else:
+            # part1 利率债 政策
+            _df_before_agg.loc[:,'group_name'] = ''
+            part_1 = _df_before_agg[_df_before_agg.em2_type == '政策银行债'].copy()
+            part_1.loc[:,'group_name'] = '1.利率债'
+            # part2 城投债
+            part_2 = _df_before_agg[(_df_before_agg.em2_type != '政策银行债') & (_df_before_agg.is_city_bond == 1)].copy()
+            part_2.loc[:,'group_name'] = '2.城投债'
+            # part3 可转债
+            part_3 = _df_before_agg[_df_before_agg.index.str.contains('转债')].join(conv_bond_info[['subject','CBSTOCKCODE','CBSTOCKNAME']].set_index('subject')).copy()
+            part_3.loc[:,'group_name'] = '3.可转债'
+            part_3.loc[:,'em2_type'] = '可转债'
+            part_3 = part_3.rename(columns={'CBSTOCKCODE':'stock_id','CBSTOCKNAME':'stock_name'})
+            # part4 其他企业信用债 a股企业债， 其他企业债
+            part_4 = _df_before_agg[(_df_before_agg.is_city_bond != 1)&(_df_before_agg.em2_type != '政策银行债')&(~_df_before_agg.index.str.contains('转债'))].copy()
+            part_4.loc[:,'stock_id'] = ''
+            part_4.loc[:,'stock_name'] = ''
+            for r in part_4.iterrows():
+                subject_name = r[0]
+                try:
+                    _df = stock_info[stock_info['name'].str.contains(subject_name)]
+                    if _df.shape[0] >= 1:
+                        part_4.loc[subject_name,'stock_id'] = _df.stock_id.values[0]
+                        part_4.loc[subject_name,'stock_name'] = _df.name.values[0]
+                        part_4.loc[subject_name,'group_name'] = '4.a股企业债'
+                    else:
+                        part_4.loc[subject_name,'group_name'] = '5.其他企业债'
+                except:
+                    part_4.loc[subject_name,'group_name'] = '5.其他企业债'
+            cols = ['group_name','bond_weight','subject','fund_name','bond_full_name','em2_type','cbl2_type','is_city_bond','stock_id','stock_name']
+            result = pd.concat([part_1,part_2,part_3,part_4]).reset_index().rename(columns={'desc_name':'fund_name'})[cols].sort_values(by=['group_name','bond_weight'],ascending=[1,0])
+            result2 = (result[['em2_type','bond_weight']].groupby('em2_type').sum() * 100).round(2).sort_values('bond_weight',ascending=False)
+            result2 = result2.astype('str') + '%'
+            result[f'{fund_type}_weight'] = result[f'{fund_type}_weight'].map(lambda x : str(round(100*x,1))+'%')
+            return result, result2
+
+
+
+
+
+    @staticmethod
+    def port_penetraded_asset_weight(res_df, m, saa):
+        res_df['weight'] = res_df.weight.map(lambda x: float(x[:-1]))
+        fund_id_list = res_df.index.tolist()
+
+        pos_weight = DataManager.basic_data(func_name=f'get_fund_hold_asset_latest', fund_list = fund_id_list)
+        fund_info = m.dts.fund_info
+        pos_weight = pos_weight.set_index('fund_id')[['stock_nav_ratio','bond_nav_ratio','cash_nav_ratio','other_nav_ratio']].fillna(0)
+        pos_weight = pos_weight.rename(columns={'stock_nav_ratio':'stock',
+                                         'bond_nav_ratio':'bond',
+                                        'cash_nav_ratio':'cash',
+                                        'other_nav_ratio':'other'})
+        pos_stats = pos_weight.T.mul(res_df['weight'].T)
+        df_weight = pd.DataFrame(pos_stats.T.sum() / pos_stats.T.sum().sum(), columns=['weights'])
+        pos_weight = pos_weight.join(fund_info[['fund_id','desc_name','wind_class_2']].set_index('fund_id')).reset_index().set_index(['fund_id','desc_name'])
+        for col in ['stock','bond','cash','other']:
+            pos_weight[col] = pos_weight[col].map(lambda x : str(round(x,1)) + '%')
+        
+        saa_df = pd.DataFrame([saa.__dict__],index=['weights']).replace(0,np.nan).dropna(axis=1).T
+        AssetPainter.double_pie_chart(df_weight,saa_df)
+        return pos_weight.sort_values('wind_class_2')
\ No newline at end of file
