@@ -14,13 +14,9 @@ _FUND_BUCKET_NAME = 'tl-fund-factors'
 
 class Factor(Calculator, metaclass=Singleton):
 
-    _factor_names_set: Set[str] = set()
 
     def __init__(self, f_name: str, f_type: Union[StockFactorType, FundFactorType], f_level: str = 'basic'):
-        assert f_name not in self._factor_names_set, f'duplicated factor with name {f_name}!!!'
         self._f_name: str = f_name
-        # 加到这个set里，可以检查是否有重名的(名字写错的)factor
-        self._factor_names_set.add(f_name)
         self._s3_file_name: str = f'{f_name}.parquet'
         self._f_level = f_level
         if isinstance(f_type, StockFactorType):
@@ -48,6 +44,13 @@ class Factor(Calculator, metaclass=Singleton):
             print(f'get data of universe {universe} failed, use default universe')
         return self._factor
 
+    # delete
+    def clear(self, recursive=False):
+        if recursive:
+            for _dep in self._deps:
+                _dep.clear(recursive)
+        self._factor = None
+
     def get_normalized(self, universe: str = 'default'):
         data = self.get(universe)
         return ((data.T - data.mean(axis=1))/data.std(axis=1)).T
@@ -64,7 +67,7 @@ class Factor(Calculator, metaclass=Singleton):
         if self._factor is None:
             return False
         self._factor.to_parquet(self._get_s3_factor_uri, compression='gzip')
-        print(f' upload to s3 success {self._f_name} {self._get_s3_factor_uri}')
+        print(f'\t[upload] to s3 success {self._f_name} {self._get_s3_factor_uri}')
         return True
 
     def calc(self):
