@@ -1,32 +1,17 @@
 from .fund_derived_factors import *
-from ..fund.derived.derived_data_helper import normalize, score_rescale
+from ...data.fund.derived.derived_data_helper import normalize, score_rescale
 
 class ScoreDataPrepare():
 
-    def fund_score_data_prepare(self):
+    def fund_ablity_prepare(self, factor_list):
+        self.factor_list = factor_list
         dt = datetime.date(2012,1,1)
-        self.fund_manager_info = FundManagerInfo().get()
         self.factor_alive = FundAlive().get().loc[dt:]
-        self.fund_pool = FilteredFundInfoWhole().get().fund_id.tolist()
-        self.factor_list = [    'AnnualRetDailyHistory', 
-                                'TotalRetDailyHistory',
-                                'MddDailyHistory',
-                                'AnnualVolDailyHistory',
-                                'DownsideStdDailyHistory',
-                                'TradeYear',
-                                'ContinueRegValue',
-                                'FundSizeCombine',
-                                'FundClAlphaHistoryWeekly',
-                                'FundClBetaHistoryWeekly',
-                                'FundPersonalHold',
-                                'RecentMonthRet',
-                            ]
         result = []
         for fac_name in self.factor_list:
             fac = eval(fac_name)().get()
             fac = fac[fac.index > dt]
-            fac = fac[fac.columns.intersection(self.fund_pool)]
-            columns = fac.columns.tolist()
+            columns = fac.columns.intersection(self.factor_alive.columns).tolist()
             fac = pd.DataFrame(fac[columns].values * self.factor_alive[columns].values, columns=columns, index=self.factor_alive.index)
             fac.columns = pd.MultiIndex.from_product([[fac_name], fac.columns])
             result.append(fac)
@@ -34,84 +19,26 @@ class ScoreDataPrepare():
             del eval(fac_name)._instances[eval(fac_name)]
         self._factor = pd.concat(result, axis=1).stack()
         self._factor = self._factor.reset_index().dropna(subset=self.factor_list, how='all')
-    
-    def fund_score_data_prepare_2(self): 
-        #self.fund_score = None
-        self.fund_score = pd.read_csv('score_df.csv',index_col=0)
-        self.fund_score = self.fund_score.drop(columns=['index'], errors='ignore').rename(columns={'level_1':'fund_id'})
-        self.fund_score = self.fund_score.set_index('datetime')
-        
         self.fund_info = FundInfo().get()
-        self.manager_info = FundManagerInfo().get().set_index('mng_id')
-        self.mng_score = MngScoreV1().get()
-        self.mng_score['mng_id'] = self.mng_score.mng_id.map(lambda x : x[x.find('_', 0) + 1 :])
-        self.mng_score = self.mng_score.set_index(['datetime','fund_type'])
-        self.mng_score = self.mng_score.sort_index()
-        self.mng_score = self.mng_score.rename(columns={'total_score':'mng_score'})
         self.fund_type = ['stock','bond','index','QDII','mmf']
         self.wind_class_type = self.fund_info.set_index('fund_id')['wind_class_2']
+        self._factor = self._factor.rename(columns={'level_1':'fund_id'})
+        self._factor.loc[:,'fund_type'] = self._factor.fund_id.map(lambda x : self.WIND_TYPE_DICT[self.wind_class_type[x]])
+        self._factor = self._factor.set_index('datetime')
 
-    def fund_score_data_append(self):
-        self.fund_score.loc[:,'fund_type'] = self.fund_score.fund_id.map(lambda x : self.WIND_TYPE_DICT[self.wind_class_type[x]])
-        date_list = sorted(self.fund_score.index.unique().tolist())
-        res = []
-        lens_dt = len(date_list)
-        _t0 = time.time()
-        for dt in date_list:
-            df_dt = self.fund_score.loc[dt].set_index('fund_type').copy()
-            mng_dic = self.manager_info[(self.manager_info.end_date >= dt)
-                                      & (self.manager_info.start_date <= dt)]['fund_id']
-            for fund_type in self.fund_type:
-                df_dt_fund_type = df_dt.loc[fund_type]
-                _mng_score = self.mng_score.loc[dt,fund_type][['mng_id','mng_score']]
-                _mng_score = _mng_score.join(mng_dic,on='mng_id')
-                _mng_score = _mng_score.groupby('fund_id').max()
-                df_dt_fund_type = df_dt_fund_type.join(_mng_score[['mng_score']], on='fund_id')
-                res.append(df_dt_fund_type)
-            dt_idx = date_list.index(dt)
-            if dt_idx % 100 == 0:
-                _t1 = time.time()
-                print(f' dt {dt} idx {dt_idx} total {lens_dt} cost {round(_t1 - _t0)}')
-                _t0 = time.time()
-        self._factor = pd.concat(res, axis=0)
-
-    def fund_score_data_prepare_v2(self):
-        dt = datetime.date(2012,1,1)
-        self.fund_manager_info = FundManagerInfo().get()
-        self.factor_alive = FundAlive().get().loc[dt:]
-        self.fund_pool = FilteredFundInfoWhole().get().fund_id.tolist()
-        self.factor_list = ['AlphaDaily1yI',
-                            'AlphaDaily3yI',
-                            'TradeYear',
-                            'FundSizeCombine',
-                            'FundClAlpha1YWeekly',
-                            'FundClBeta1YWeekly']
-        result = []
-        for fac_name in self.factor_list:
-            fac = eval(fac_name)().get()
-            fac = fac[fac.index > dt]
-            fac = fac[fac.columns.intersection(self.fund_pool)]
-            columns = fac.columns.tolist()
-            fac = pd.DataFrame(fac[columns].values * self.factor_alive[columns].values, columns=columns, index=self.factor_alive.index)
-            fac.columns = pd.MultiIndex.from_product([[fac_name], fac.columns])
-            result.append(fac)
-            del fac
-            del eval(fac_name)._instances[eval(fac_name)]
-        self.fund_score = pd.concat(result, axis=1).stack()
-        self.fund_score = self.fund_score.reset_index().dropna(subset=self.factor_list, how='all')
-    
-    def fund_score_data_prepare_2_v2(self): 
-        self.fund_score = self.fund_score.drop(columns=['index'], errors='ignore').rename(columns={'level_1':'fund_id'})
-        self.fund_score = self.fund_score.set_index('datetime')
+    def manager_score_to_fund_prepare(self):
+        self.total_score = MngScoreV1().get().reset_index()
+        self.fund_manager_info = FundManagerInfo().get().set_index('fund_id')[['mng_id','start_date','end_date']]
+        self.fund_nav = FundNavDaily().get()
         self.fund_info = FundInfo().get()
-        self.manager_info = FundManagerInfo().get().set_index('mng_id')
-        self.mng_score = MngScoreV2().get()
-        self.mng_score['mng_id'] = self.mng_score.mng_id.map(lambda x : x[x.find('_', 0) + 1 :])
-        self.mng_score = self.mng_score.set_index(['datetime','fund_type'])
-        self.mng_score = self.mng_score.rename(columns={'total_score':'mng_score'})
-        self.mng_score = self.mng_score.sort_index()
-        self.fund_type = ['stock','bond','index','QDII']
-        self.wind_class_type = self.fund_info.set_index('fund_id')['wind_class_2']
+        type_list = ['stock','bond','index','QDII','mmf']
+        self.manager_score = {}
+        for type_i in type_list:
+            _df = self.total_score[self.total_score['fund_type'] == type_i].reset_index().rename(columns={'index':'datetime'})
+            _df = _df.pivot_table(index='datetime',columns='mng_id',values='mng_score')
+            self.manager_score[type_i] = _df
+        self.date_index = self.manager_score['stock'].index
+        self.wind_class_dict = self.fund_info.set_index('fund_id').to_dict()['wind_class_2']
 
     def manager_score_data_prepare(self):
         dt = datetime.date(2012,1,1)
@@ -134,32 +61,53 @@ class ScoreDataPrepare():
             del fac
             del eval(fac_name)._instances[eval(fac_name)]
         self._factor = pd.concat(result, axis=1).stack()
-        self._factor = self._factor.reset_index().dropna(subset=self.factor_list, how='all')
-        self._factor.loc[:,'fund_type'] = self._factor.mng_id.map(lambda x: x.split('_')[0])
-        self._factor = self._factor.set_index(['datetime','fund_type'])
-        self.fund_type = ['stock','bond','index','QDII','mmf']
-        
-    def manager_score_data_prepare_v2(self):
-        dt = datetime.date(2012,1,1)
-        self.factor_list = ['MngAnnualRetDaily1Y',
-                            'MngAnnualRetDaily3Y',
-                            'MngFundTypeTradingDays',
-                            'MngClAlphaWeekly1Y',
-                            'MngClBetaWeekly1Y',
-                            'MngFundSize']
-        result = []
-        for fac_name in self.factor_list:
-            fac = eval(fac_name)().get()
-            fac = fac[fac.index > dt]
-            fac.columns = pd.MultiIndex.from_product([[fac_name], fac.columns])
-            result.append(fac)
-            del fac
-            del eval(fac_name)._instances[eval(fac_name)]
-        self._factor = pd.concat(result, axis=1).stack()
         self._factor = self._factor.reset_index().dropna(subset=self.factor_list, how='all').rename(columns={'level_1':'mng_id'})
         self._factor.loc[:,'fund_type'] = self._factor.mng_id.map(lambda x: x.split('_')[0])
         self._factor = self._factor.set_index(['datetime','fund_type'])
         self.fund_type = ['stock','bond','index','QDII','mmf']
+            
+    def score_rescale(self, df):
+        return df.rank(pct=True) * 100
+
+    def process_score(self, df):
+        df = df.reset_index().set_index('fund_id').drop(columns=['fund_type'])
+        return df.rank(pct=True) * 100
+
+    def ability_calc(self):
+        date_list = sorted(self._factor.index.unique().tolist())
+        res = []
+        lens_dt = len(date_list)
+        _t0 = time.time()
+        for dt in date_list:
+            df_dt = self._factor.loc[dt].set_index('fund_type').copy()
+            for fund_type in self.fund_type:
+                try:
+                    df_dt_fund_type = df_dt.loc[fund_type]
+                except:
+                    continue
+                res.append(self.calc_fund(df_dt_fund_type, fund_type, dt))
+            dt_idx = date_list.index(dt)
+            if dt_idx % 100 == 0:
+                _t1 = time.time()
+                print(f' dt {dt} idx {dt_idx} total {lens_dt} cost {round(_t1 - _t0)}')
+                _t0 = time.time()
+        res = [i for i in res if i is not None]
+        self._factor = pd.concat(res, axis=0)
+        self._factor = self._factor.reset_index().pivot_table(index='datetime',values='fund_score',columns='fund_id')
+
+    def score_fund(self, df, fund_type):
+        return pd.DataFrame()
+
+    def calc_fund(self, df, fund_type, dt):
+        df = self.process_score(df)
+        if df is None:
+            return None
+        df = self.score_fund(df, fund_type)
+        if df is None:
+            return None
+        df.loc[:,'fund_type'] = fund_type
+        df.loc[:,'datetime'] = dt
+        return df.dropna(subset=['fund_score'])
 
 class MngScoreV1(Factor, ScoreDataPrepare):
     # 雷达图基金评价 历史业绩为重
@@ -218,7 +166,7 @@ class MngScoreV1(Factor, ScoreDataPrepare):
         return df[['mng_id','mng_ret_ability','mng_experience','mng_score']]
 
     def calc(self):
-        #self.manager_score_data_prepare()
+        self.manager_score_data_prepare()
         date_list = sorted(self._factor.index.get_level_values(0).unique().tolist())
         result = []
         _t0 = time.time()
@@ -237,207 +185,170 @@ class MngScoreV1(Factor, ScoreDataPrepare):
         self._factor = pd.concat(result).dropna(subset=['mng_score'])
         self._factor = self._factor.reset_index()
 
-class MngScoreV2(Factor, ScoreDataPrepare):
-    
-    def __init__(self):
-        super().__init__(f_name='MngScoreV2', f_type=FundFactorType.DERIVED, f_level='score')
-
-    def calc_mng(self, df, fund_type, dt):
-        df = self.process_score(df)
-        df = self.score_mng(df)
-        df.loc[:,'fund_type'] = fund_type
-        df.loc[:,'datetime'] = dt
-        return df
-
-    def score_mng(self, df):
-        df.loc[:,'mng_score'] = self.score_rescale(0.3*df.MngAnnualRetDaily1Y+0.3*df.MngAnnualRetDaily3Y+0.1*df.MngClAlphaWeekly1Y+0.1*df.MngClBetaWeekly1Y+0.15*df.MngFundSize+0.15*df.MngFundTypeTradingDays)
-        return df
-
-    def score_rescale(self, df):
-        return df.rank(pct=True) * 100
-
-    def process_score(self, df):
-        df = df.reset_index().set_index('mng_id').drop(columns=['datetime','fund_type'])
-        return df.rank(pct=True) * 100
-    
-    def calc(self):
-        self.manager_score_data_prepare_v2()
+    def append_update(self):
+        self._exsited_factor = self.get()
+        last_date = pd.to_datetime(self._exsited_factor.index[-1]).date()
+        self.manager_score_data_prepare()
         date_list = sorted(self._factor.index.get_level_values(0).unique().tolist())
+        date_list = [pd.to_datetime(i).date() for i in date_list]
+        date_list = [i for i in date_list if i > last_date]
         result = []
-        _t0 = time.time()
+        if len(date_list)  == 0:
+            return True
         for dt in date_list:
-            #df_dt = self._factor.loc[dt]
             for fund_type in self.fund_type:
                 df_i = self._factor.loc[dt,fund_type].copy()
-                df_i = self.calc_mng(df_i, fund_type, dt)
+                df_i = self.calc_mng(df_i, fund_type)
                 result.append(df_i)
             dt_idx = date_list.index(dt)
-            if dt_idx % 100 == 0:
-                print(f'dt {dt} idx {dt_idx}')
-
-        _t1 = time.time()
-        print(_t1 - _t0)
+        if len(result) == 0:
+            return True
         self._factor = pd.concat(result).dropna(subset=['mng_score'])
-        self._factor = self._factor.reset_index()
+        self._factor = self._factor.reset_index().set_index('datetime')
+        self._factor = self._exsited_factor.append(self._factor)
+        td = self._factor.index.tolist()
+        td = pd.to_datetime(td)
+        td = [i.date() for i in td] 
+        self._factor.index = td
+        return self.save()
+
+class FundMngScoreV1(Factor, ScoreDataPrepare):
+
+    def __init__(self):
+        super().__init__(f_name='FundMngScoreV1', f_type=FundFactorType.DERIVED, f_level='score')
+
+    def loop_item(self, fund_id):
+        wind_type = self.wind_class_dict.get(fund_id)
+        fund_type = self.WIND_TYPE_DICT.get(wind_type, 'stock')
+        if fund_id not in self.fund_manager_info.index:
+            return None
+        single_fund_info = self.fund_manager_info.loc[[fund_id]]
+        result = []
+        for row in single_fund_info.itertuples(index=False):
+            if row.mng_id not in self.manager_score[fund_type]:
+                continue
+            manager_score = pd.Series((self.date_index >= row.start_date) & (self.date_index <= row.end_date), index=self.date_index,name=row.mng_id) * 1
+            manager_score = manager_score * self.manager_score[fund_type][row.mng_id]
+            result.append(manager_score)
+        if result  == []:
+            return None
+        fund_manager_score_i = pd.concat(result,axis=1).max(axis=1)
+        fund_manager_score_i.name = fund_id
+        return fund_manager_score_i
+
+    def calc(self):
+        self.manager_score_to_fund_prepare() 
+        score_result_total = []
+        fund_list = self.fund_nav.columns.tolist()
+        _t0 = time.time()
+        for fund_id in fund_list:
+            res_i = self.loop_item(fund_id)
+            if res_i is not None:
+                score_result_total.append(res_i)
+            idx = fund_list.index(fund_id)
+            if idx % 1000 == 0:
+                _t1 = time.time()
+                #print(f'fund_id {fund_id} {idx} {len(fund_list)} cost { round(_t1 - _t0,2)}')
+                _t0 = time.time()
+        self._factor = pd.concat(score_result_total, axis=1)
+        self.data = None
+
+class RetAbilityFundScore(Factor, ScoreDataPrepare):
+
+    def __init__(self):
+        super().__init__(f_name='RetAbilityFundScore', f_type=FundFactorType.DERIVED, f_level='score')
+
+    def score_fund(self, df, fund_type):
+        if fund_type in ['stock','index','QDII']:
+            df.loc[:,'fund_score'] = self.score_rescale(0.4 * df.AnnualRetDailyHistory + 0.6 * df.TotalRetDailyHistory)
+        elif fund_type in ['bond']:
+            df.loc[:,'fund_score'] = self.score_rescale(0.8 * df.AnnualRetDailyHistory + 0.2 * df.TotalRetDailyHistory)
+        elif fund_type in ['mmf']:
+            df.loc[:,'fund_score'] = self.score_rescale(df.RecentMonthRet)
+        return df[['fund_score']]
+
+    def calc(self):
+        fund_list = ['AnnualRetDailyHistory', 
+                     'TotalRetDailyHistory',
+                     'RecentMonthRet']
         
-class FundScoreV1(Factor, ScoreDataPrepare):
+        self.fund_ablity_prepare(fund_list)
+        self.ability_calc()
+
+class RiskAbilityFundScore(Factor, ScoreDataPrepare):
 
     def __init__(self):
-        super().__init__(f_name='FundScoreV1', f_type=FundFactorType.DERIVED, f_level='score')
+        super().__init__(f_name='RiskAbilityFundScore', f_type=FundFactorType.DERIVED, f_level='score')
 
-    def calc_fund(self, df, fund_type, dt):
-        df = self.process_score(df)
-        if fund_type == 'stock':
-            df = self.score_fund_stock(df)
-        if fund_type == 'bond':
-            df = self.score_fund_bond(df)
-        if fund_type == 'index':
-            df = self.score_fund_index(df)
-        if fund_type == 'QDII':
-            df = self.score_fund_qdii(df)
-        if fund_type == 'mmf':
-            df = self.score_fund_mmf(df)
-        df.loc[:,'fund_type'] = fund_type
-        df.loc[:,'datetime'] = dt
-        return df
-
-    def score_rescale(self, df):
-        return df.rank(pct=True) * 100
-
-    def process_score(self, df):
-        df = df.reset_index().set_index('fund_id').drop(columns=['datetime','fund_type'])
-        return df.rank(pct=True) * 100
-
-    def score_fund_stock(self, df):
-        ret_ability = self.score_rescale(0.4 * df.AnnualRetDailyHistory + 0.6 * df.TotalRetDailyHistory)
-        risk_ability = self.score_rescale(0.3 * -df.MddDailyHistory + 0.4 * -df.AnnualVolDailyHistory + 0.3 * -df.DownsideStdDailyHistory)
-        stable_ability = self.score_rescale(0.4 * df.TradeYear + 0.4 * df.ContinueRegValue + 0.2 * df.FundSizeCombine)
-        select_time = df.FundClBetaHistoryWeekly
-        select_stock = df.FundClAlphaHistoryWeekly
-        #df['mng_score'] = df.mng_score
-        df.loc[:,'fund_score'] = self.score_rescale(ret_ability + risk_ability + stable_ability + select_time + select_stock)
-        df.loc[:,'total_score'] = self.score_rescale(5/6 * df.fund_score + 1/6 * df.mng_score)
-        return df[['fund_score','mng_score','total_score']]
-
-    def score_fund_bond(self, df):
-        ret_ability = self.score_rescale(0.8 * df.AnnualRetDailyHistory + 0.2 * df.TotalRetDailyHistory)
-        risk_ability = self.score_rescale(0.3 * -df.MddDailyHistory + 0.4 * -df.AnnualVolDailyHistory + 0.3 * -df.DownsideStdDailyHistory)
-        stable_ability = self.score_rescale(0.4 * df.TradeYear + 0.4 * df.ContinueRegValue + 0.2 * df.FundSizeCombine)
-        select_time = df.FundClBetaHistoryWeekly
-        select_stock = df.FundClAlphaHistoryWeekly
-        #df['mng_score'] = df.mng_score
-        df.loc[:,'fund_score'] = self.score_rescale(ret_ability + risk_ability + stable_ability + select_time + select_stock)
-        df.loc[:,'total_score'] = self.score_rescale(5/6 * df.fund_score + 1/6 * df.mng_score)
-        return df[['fund_score','mng_score','total_score']]
-
-    def score_fund_index(self, df):
-        ret_ability = self.score_rescale( 0.4 * df.AnnualRetDailyHistory + 0.6 * df.TotalRetDailyHistory)
-        risk_ability = self.score_rescale(0.3 * -df.MddDailyHistory + 0.4 * -df.AnnualVolDailyHistory + 0.3 * -df.DownsideStdDailyHistory)
-        stable_ability = self.score_rescale(0.4 * df.TradeYear + 0.4 * df.ContinueRegValue + 0.2 * df.FundSizeCombine)
-        #df['mng_score'] = df.mng_score
-        df.loc[:,'fund_score'] = self.score_rescale(ret_ability + risk_ability + stable_ability)
-        df.loc[:,'total_score'] = self.score_rescale(3/4 * df.fund_score + 1/4 * df.mng_score)
-        return df[['fund_score','mng_score','total_score']]
-
-    def score_fund_qdii(self, df):
-        ret_ability = self.score_rescale( 0.4 * df.AnnualRetDailyHistory + 0.6 * df.TotalRetDailyHistory)
-        risk_ability = self.score_rescale(0.3 * -df.MddDailyHistory + 0.4 * -df.AnnualVolDailyHistory + 0.3 * -df.DownsideStdDailyHistory)
-        stable_ability = self.score_rescale(0.4 * df.TradeYear + 0.4 * df.ContinueRegValue + 0.2 * df.FundSizeCombine)
-        #df['mng_score'] = df.mng_score
-        df.loc[:,'fund_score'] = self.score_rescale(ret_ability + risk_ability + stable_ability)
-        df.loc[:,'total_score'] = self.score_rescale(3/4 * df.fund_score + 1/4 * df.mng_score)
-        return df[['fund_score','mng_score','total_score']]
-
-    def score_fund_mmf(self, df):
-        ret_ability = df.RecentMonthRet
-        risk_ability = self.score_rescale(df.FundPersonalHold + df.FundSizeCombine)
-        df.loc[:,'fund_score'] = self.score_rescale(ret_ability + risk_ability)
-        #df['mng_score'] = df.mng_score
-        df.loc[:,'total_score'] = self.score_rescale(2/3 * df.fund_score + 1/3 * df.mng_score)
-        return df[['fund_score','mng_score','total_score']]
+    def score_fund(self, df, fund_type):
+        if fund_type in ['stock','index','QDII','bond']:
+            df.loc[:,'fund_score'] = self.score_rescale(0.3 * df.MddDailyHistory + 0.4 * df.AnnualVolDailyHistory + 0.3 * df.DownsideStdDailyHistory)
+        elif fund_type in ['mmf']:
+            df.loc[:,'fund_score'] = self.score_rescale(0.5 * df.FundPersonalHold + 0.5 * df.FundSizeCombine)
+        return df[['fund_score']]
 
     def calc(self):
-        self.fund_score_data_prepare()
-        self.fund_score_data_prepare_2()
-        self.fund_score_data_append()
-        date_list = sorted(self.fund_score.index.unique().tolist())
-        res = []
-        lens_dt = len(date_list)
-        _t0 = time.time()
-        for dt in date_list:
-            df_dt = self.fund_score.loc[dt].copy()
-            df_dt.loc[:,'fund_type'] = df_dt.fund_id.map(lambda x : self.WIND_TYPE_DICT[self.wind_class_type[x]])
+        fund_list = ['MddDailyHistory', 
+                     'AnnualVolDailyHistory',
+                     'DownsideStdDailyHistory',
+                     'FundPersonalHold',
+                     'FundSizeCombine']
+        self.fund_ablity_prepare(fund_list)
+        self.ability_calc()
 
-            _end_date = pd.to_datetime(dt).date()
-            mng_dic = self.manager_info[(self.manager_info.end_date >= _end_date)
-                                    & (self.manager_info.start_date <= _end_date)]['fund_id']
-            _res = []
-            mng_score = self.mng_score.loc[dt][['mng_id','mng_score','fund_type']].set_index('fund_type')
-            for fund_type in self.fund_type:
-                df_dt_fund_type = df_dt[df_dt['fund_type'] == fund_type]
-                _mng_score = mng_score.loc[fund_type]
-                _mng_score = _mng_score.join(mng_dic,on='mng_id')
-                _mng_score = _mng_score.reset_index().groupby('fund_id').max()
-                df_dt_fund_type = df_dt_fund_type.join(_mng_score[['mng_score']], on='fund_id')
-                _res.append(self.calc_fund(df_dt_fund_type, fund_type, dt))
-            res.append(pd.concat(_res))
-            dt_idx = date_list.index(dt)
-            if dt_idx % 100 == 0:
-                _t1 = time.time()
-                print(f' dt {dt} idx {dt_idx} total {lens_dt} cost {round(_t1 - _t0)}')
-                _t0 = time.time()
-        self._factor = pd.concat(res)
+class StableAbility(Factor, ScoreDataPrepare):
 
-class FundScoreV2(Factor, ScoreDataPrepare):
-    
     def __init__(self):
-        super().__init__(f_name='FundScoreV2', f_type=FundFactorType.DERIVED, f_level='score')
+        super().__init__(f_name='StableAbility', f_type=FundFactorType.DERIVED, f_level='score')
 
-    def calc_fund(self, df, fund_type, dt):
-        df = self.process_score(df)
-        df = self.score_fund(df)
-        df.loc[:,'fund_type'] = fund_type
-        df.loc[:,'datetime'] = dt
-        return df.dropna(subset=['total_score'])
-
-    def score_rescale(self, df):
-        return df.rank(pct=True) * 100
-
-    def process_score(self, df):
-        df = df.reset_index().set_index('fund_id').drop(columns=['fund_type'])
-        return df.rank(pct=True) * 100
-
-    def score_fund(self, df):
-        df.loc[:,'fund_score'] = self.score_rescale(0.3*df.AlphaDaily1yI+0.3*df.AlphaDaily3yI+0.1*df.FundClAlpha1YWeekly+0.1*df.FundClBeta1YWeekly+0.15*df.FundSizeCombine+0.15*df.TradeYear)
-        df.loc[:,'total_score'] = self.score_rescale(3/4 * df.fund_score + 1/4 * df.mng_score)
-        return df[['fund_score','mng_score','total_score']]
+    def score_fund(self, df, fund_type):
+        if fund_type in ['stock','index','QDII','bond']:
+            df.loc[:,'fund_score'] = self.score_rescale(0.4 * df.TradeYear + 0.4 * df.ContinueRegValue + 0.2 * df.FundSizeCombine)
+        else:
+            return None
+        return df[['fund_score']]
 
     def calc(self):
-        self.fund_score_data_prepare_v2()
-        self.fund_score_data_prepare_2_v2()
-        self.fund_score.loc[:,'fund_type'] = self.fund_score.fund_id.map(lambda x : self.WIND_TYPE_DICT[self.wind_class_type[x]])
-        date_list = sorted(self.fund_score.index.unique().tolist())
-        res = []
-        lens_dt = len(date_list)
-        _t0 = time.time()
-        for dt in date_list:
-            df_dt = self.fund_score.loc[dt].set_index('fund_type').copy()
-            mng_dic = self.manager_info[(self.manager_info.end_date >= dt)
-                                      & (self.manager_info.start_date <= dt)]['fund_id']
-            for fund_type in self.fund_type:
-                df_dt_fund_type = df_dt.loc[fund_type]
-                _mng_score = self.mng_score.loc[dt,fund_type][['mng_id','mng_score']]
-                _mng_score = _mng_score.join(mng_dic,on='mng_id')
-                # delete useless
-                _mng_score = _mng_score.groupby('fund_id').max()
-                df_dt_fund_type = df_dt_fund_type.join(_mng_score[['mng_score']], on='fund_id')
-                res.append(self.calc_fund(df_dt_fund_type, fund_type, dt))
-            dt_idx = date_list.index(dt)
-            if dt_idx % 100 == 0:
-                _t1 = time.time()
-                print(f' dt {dt} idx {dt_idx} total {lens_dt} cost {round(_t1 - _t0)}')
-                _t0 = time.time()
-        self._factor = pd.concat(res, axis=0)
+        fund_list = ['TradeYear', 
+                     'ContinueRegValue',
+                     'FundSizeCombine']
+        
+        self.fund_ablity_prepare(fund_list)
+        self.ability_calc()
+
+class SelectTimeAbility(Factor, ScoreDataPrepare):
+
+    def __init__(self):
+        super().__init__(f_name='SelectTimeAbility', f_type=FundFactorType.DERIVED, f_level='score')
+
+    def score_fund(self, df, fund_type):
+        if fund_type in ['stock','QDII']:
+            df.loc[:,'fund_score'] = self.score_rescale(df.FundClBetaHistoryWeekly)
+        else:
+            return None
+        return df[['fund_score']]
+
+    def calc(self):
+        fund_list = ['FundClBetaHistoryWeekly']
+        self.fund_ablity_prepare(fund_list)
+        self.ability_calc()
+
+class SelectStockAbility(Factor, ScoreDataPrepare):
+
+    def __init__(self):
+        super().__init__(f_name='SelectStockAbility', f_type=FundFactorType.DERIVED, f_level='score')
+
+    def score_fund(self, df, fund_type):
+        if fund_type in ['stock','QDII']:
+            df.loc[:,'fund_score'] = self.score_rescale(df.FundClAlphaHistoryWeekly)
+        else:
+            return None
+        return df[['fund_score']]
+
+    def calc(self):
+        fund_list = ['FundClAlphaHistoryWeekly']
+        self.fund_ablity_prepare(fund_list)
+        self.ability_calc()
 
 class UpdateDerivedScoreStart:
     pass
