@@ -11,6 +11,7 @@ import statsmodels.api as sm
 from statsmodels.tsa.ar_model import AutoReg
 from scipy.stats import kurtosis, skew
 from decimal import Decimal, ROUND_HALF_UP, Context
+from pandas.tseries.offsets import DateOffset
 
 class CalculatorBase:
 
@@ -26,6 +27,27 @@ class CalculatorBase:
         '深交所主板': 500,
     }
 
+    @staticmethod
+    def get_begin_date_dic(week_end_date):
+        begin_date_period_dic = {
+            'w1': DateOffset(weeks=1),
+            'm1': DateOffset(months=1),
+            'm3': DateOffset(months=3),
+            'm6': DateOffset(months=6),
+            'y1': DateOffset(years=1),
+            'y3': DateOffset(years=3),
+            'y5': DateOffset(years=5),
+        }
+        last_date = datetime.datetime.now().date()
+        begin_date_dic = {}
+        for name, value in begin_date_period_dic.items():
+            if name == 'w1':
+                bd = (week_end_date - value).date()
+            else:
+                bd = (last_date - value).date()
+            begin_date_dic[name] = bd
+        return last_date, begin_date_dic
+
     @staticmethod
     def calc_cl_alpha_beta(total: np.ndarray):
         if total.shape[0] <= 1:
@@ -91,11 +113,15 @@ class CalculatorBase:
                         values:            pd.core.series.Series,
                         benchmark_values:  pd.core.series.Series=None,
                         risk_free_rate:    float=0.025,
-                        frequency:         str='1D'):# '1W' '1M'
+                        frequency:         str='1D',
+                        ret_method:        str='log_ret',
+                        ):# '1W' '1M'
         assert frequency in ['1D','1W','2W','1M'], 'frequency provided is not included, must in 1D 1W 2W 1M'
-        risk_free_per_day = risk_free_rate / CalculatorBase.TRADING_DAYS_PER_YEAR
+        assert ret_method in ['log_ret', 'pct_ret'], 'ret_method is not included'
         dates = pd.to_datetime(dates)
         dates = [i.date() for i in dates]
+        _end_dt = dates[-1]
+        last_date, begin_date_dic = CalculatorBase.get_begin_date_dic(week_end_date = _end_dt)
         assert len(dates) == len(values), 'date and port_values has different numbers'
         if len(dates) > 2:
             sr = pd.Series(values, index=dates).sort_index().dropna()
@@ -108,6 +134,7 @@ class CalculatorBase:
                 period_1y = CalculatorBase.TRADING_DAYS_PER_YEAR
                 period_3y = period_1y * 3
                 period_5y = period_1y * 5
+                risk_free_per_period = risk_free_rate / CalculatorBase.TRADING_DAYS_PER_YEAR
             elif frequency == '1W':
                 period_1m = 4
                 period_3m = period_1m * 3
@@ -115,13 +142,14 @@ class CalculatorBase:
                 period_1y = CalculatorBase.TOTAL_WEEKS_PER_YEAR
                 period_3y = period_1y * 3
                 period_5y = period_1y * 5
+                risk_free_per_period = risk_free_rate / 52
             elif frequency == '2W':
                 period_1m = 2
                 period_3m = period_1m * 3
                 period_6m = period_1m * 6
                 period_1y = CalculatorBase.TOTAL_WEEKS_PER_YEAR / 2
                 period_3y = period_1y * 3
-                period_5y = period_1y * 5
+                risk_free_per_period = risk_free_rate / 26
             elif frequency == '1M':
                 period_1m = 1
                 period_3m = period_1m * 3
@@ -129,7 +157,12 @@ class CalculatorBase:
                 period_1y = CalculatorBase.TOTAL_MONTHS_PER_YEAR
                 period_3y = period_1y * 3
                 period_5y = period_1y * 5
-            sr_ret = np.log(sr).diff(1).iloc[1:]
+                risk_free_per_period = risk_free_rate / 12
+            if ret_method == 'log_ret':
+                sr_ret = np.log(sr).diff(1).iloc[1:]
+            elif ret_method == 'pct_ret':
+                sr_ret = sr.pct_change(1).iloc[1:]
+
             start_date = sr.index[0] 
             end_date = sr.index[-1]
             days = (dates[-1] - dates[0]).days
@@ -138,12 +171,14 @@ class CalculatorBase:
             trade_year = days / CalculatorBase.TOTAL_DAYS_PER_YEAR
             annual_ret = math.pow(last_unit_nav, CalculatorBase.TOTAL_DAYS_PER_YEAR/days) - 1
             annual_vol = sr_ret.std(ddof=1) * np.sqrt(period_1y)
-            recent_1m_ret = sr[-1] / sr[-(period_1m + 1)] - 1 if len(sr) > (period_1m + 1) else sr[-1] / sr[0] - 1
-            recent_3m_ret = sr[-1] / sr[-(period_3m + 1)] - 1 if len(sr) > (period_3m + 1) else sr[-1] / sr[0] - 1
-            recent_6m_ret = sr[-1] / sr[-(period_6m + 1)] - 1 if len(sr) > (period_6m + 1) else sr[-1] / sr[0] - 1
-            recent_1y_ret = sr[-1] / sr[-(period_1y + 1)] - 1 if len(sr) > (period_1y + 1) else sr[-1] / sr[0] - 1
-            recent_3y_ret = sr[-1] / sr[-(period_3y + 1)] - 1 if len(sr) > (period_3y + 1) else sr[-1] / sr[0] - 1
-            recent_5y_ret = sr[-1] / sr[-(period_5y + 1)] - 1 if len(sr) > (period_5y + 1) else sr[-1] / sr[0] - 1
+            _begin_date = sr.index[0]
+            recent_1w_ret = sr.loc[:last_date][-1] / sr.loc[:max(begin_date_dic['w1'],_begin_date)][-1] - 1
+            recent_1m_ret = sr.loc[:last_date][-1] / sr.loc[:max(begin_date_dic['m1'],_begin_date)][-1] - 1
+            recent_3m_ret = sr.loc[:last_date][-1] / sr.loc[:max(begin_date_dic['m3'],_begin_date)][-1] - 1
+            recent_6m_ret = sr.loc[:last_date][-1] / sr.loc[:max(begin_date_dic['m6'],_begin_date)][-1] - 1
+            recent_1y_ret = sr.loc[:last_date][-1] / sr.loc[:max(begin_date_dic['y1'],_begin_date)][-1] - 1
+            recent_3y_ret = sr.loc[:last_date][-1] / sr.loc[:max(begin_date_dic['y3'],_begin_date)][-1] - 1
+            recent_5y_ret = sr.loc[:last_date][-1] / sr.loc[:max(begin_date_dic['y5'],_begin_date)][-1] - 1
             worst_3m_ret = sr.pct_change(period_3m).min()
             worst_6m_ret = sr.pct_change(period_6m).min()
             last_mv_diff = sr[-1] - sr[-2]
@@ -182,7 +217,10 @@ class CalculatorBase:
                 sr_benchmark = pd.Series(benchmark_values, index=dates).sort_index().dropna()
                 sr_benchmark = sr_benchmark.set_axis(pd.to_datetime(sr_benchmark.index), inplace=False).resample(frequency).last().dropna()
                 sr_benchmark.index = [i.date() for i in sr_benchmark.index]
-                sr_benchmark_ret = np.log(sr_benchmark).diff(1).iloc[1:]
+                if ret_method == 'log_ret':
+                    sr_benchmark_ret = np.log(sr_benchmark).diff(1).iloc[1:]
+                elif ret_method == 'pct_ret':
+                    sr_benchmark_ret = sr_benchmark.pct_change(1).iloc[1:]
                 last_unit_benchmark = sr_benchmark[-1] / sr_benchmark[0]
                 annual_ret_benchmark = math.pow(last_unit_benchmark, CalculatorBase.TOTAL_DAYS_PER_YEAR / days) - 1
                 df = pd.concat([sr, sr_benchmark],axis=1)
@@ -191,7 +229,7 @@ class CalculatorBase:
                 df_monthly.index = df_monthly.index.date
                 monthly_ret = df_monthly.pct_change(1).dropna()
                 continue_value_m = CalculatorBase.calc_continue_regress_v(monthly_ret.fund_ret,risk_free_rate)
-                df_ret_risk_free = (df.pct_change(1) - risk_free_per_day).dropna()
+                df_ret_risk_free = (df.pct_change(1) - risk_free_per_period).dropna()
                 cl_res = CalculatorBase.calc_cl_alpha_beta(df_ret_risk_free.to_numpy())
                 alpha_cl = cl_res['alpha'] * period_1y
                 beta_cl = cl_res['beta']
@@ -208,16 +246,16 @@ class CalculatorBase:
                 corr = sr_ret.corr(sr_benchmark_ret)    
                 track_err = (sr_ret - sr_benchmark_ret).std(ddof=1) * np.sqrt(period_1y)
                 info = excess_ret / track_err
-                x = sr_benchmark_ret.dropna().values
-                y = sr_ret.dropna().values
+                x = (sr_benchmark_ret - risk_free_per_period).dropna() .values
+                y = (sr_ret - risk_free_per_period).dropna().values
                 x = sm.add_constant(x)
                 model = regression.linear_model.OLS(y,x).fit()
                 alpha = model.params[0]
                 beta = model.params[1]
                 treynor = (annual_ret - risk_free_rate) / beta
-                not_system_risk = sum((sr_ret - risk_free_per_day)**2) \
-                                - alpha * sum(sr_ret - risk_free_per_day) \
-                                - beta * sum((sr_ret - risk_free_per_day) * (sr_benchmark_ret - risk_free_per_day))
+                not_system_risk = sum((sr_ret - risk_free_per_period)**2) \
+                                - alpha * sum(sr_ret - risk_free_per_period) \
+                                - beta * sum((sr_ret - risk_free_per_period) * (sr_benchmark_ret - risk_free_per_period))
                 not_system_risk = np.sqrt(not_system_risk / (len(sr_ret) - 2))
                 alpha = alpha * period_1y
                 df_ret = np.log(df).diff(1).iloc[1:]
@@ -245,6 +283,7 @@ class CalculatorBase:
                     'annual_ret':annual_ret,
                     'annual_vol':annual_vol,
                     'sharpe':sharpe,
+                    'recent_1w_ret':recent_1w_ret,
                     'recent_1m_ret':recent_1m_ret,
                     'recent_3m_ret':recent_3m_ret,
                     'recent_6m_ret':recent_6m_ret,
@@ -300,6 +339,7 @@ class CalculatorBase:
                     'annual_ret':annual_ret,
                     'annual_vol':annual_vol,
                     'sharpe':sharpe,
+                    'recent_1w_ret':recent_1w_ret,
                     'recent_1m_ret':recent_1m_ret,
                     'recent_3m_ret':recent_3m_ret,
                     'recent_6m_ret':recent_6m_ret,
@@ -336,6 +376,7 @@ class CalculatorBase:
                     'annual_ret':0,
                     'annual_vol':0,
                     'sharpe':0,
+                    'recent_1w_ret':0,
                     'recent_1m_ret':0,
                     'recent_3m_ret':0,
                     'recent_6m_ret':0,
