@@ -2,6 +2,9 @@
 from typing import Dict, List, Set, Optional
 import requests
 import json
+
+import pandas as pd
+
 from .config import SurfingConfigurator
 
 
@@ -20,6 +23,7 @@ class WechatBot(object):
             }
         }
         res = requests.post(url=self.wechat_webhook, data=json.dumps(message), timeout=20)
+        print(res)
 
     def send_data_update_status(self, dt, failed_tasks, updated_count):
         # Send markdown content
@@ -75,3 +79,33 @@ class WechatBot(object):
         else:
             markdown_content += f'\n><font color=\"info\">更新所有factor成功</font>'
         self.send(markdown_content)
+
+    def send_hedge_fund_nav_update(self, df: pd.DataFrame):
+        markdown_content = '#### <font color=\"info\">更新以下私募基金净值数据成功</font>'
+        markdown_content += f'\n>{df.to_markdown()}'
+        self.send(markdown_content)
+
+    def send_hedge_fund_nav_update_failed(self, err_msg: str):
+        markdown_content = '#### <font color=\"warning\">更新私募基金净值数据错误！！</font>'
+        markdown_content += f'\n>err msg: {err_msg}'
+        self.send(markdown_content)
+
+    def send_fof_nav_update(self):
+        self.send(markdown_content)
+
+    def send_fof_nav_update_failed(self, err_msg: str):
+        self.send(markdown_content)
+
+    def send_refinancing_key_info(self, dt, stocks, key):
+        if stocks:
+            markdown_content = f'#### {dt}'
+            markdown_content += f'\n下列上市公司发布再融资预案，发行对象中包括 **<font color=\"warning\">{key}</font>**'
+            markdown_content += f'\n>**<font color=\"info\">{stocks}</font>**'
+            self.send(markdown_content)
+
+    def send_refinancing_impl_key_info(self, dt, stocks, key):
+        if stocks:
+            markdown_content = f'#### {dt}'
+            markdown_content += f'\n下列上市公司发布再融资实施公告，获配对象中包括 **<font color=\"warning\">{key}</font>**'
+            markdown_content += f'\n>**<font color=\"info\">{stocks}</font>**'
+            self.send(markdown_content)
